/* This file is part of the Palabos library.
   Copyright (C) 2009, 2010 Jonas Latt
   E-mail contact: jonas@lbmethod.org
   The most recent release of Palabos can be downloaded at
   <http://www.lbmethod.org/palabos/>

   The library Palabos is free software: you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation, either version 3 of the
   License, or (at your option) any later version.

   The library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   Datas :

   Total length = L_tot = 9 mm
   Pipe length  = L_Pip = 7 mm
   L_inlet  = 10mm
   Diameter = 10mm
   One cavity = 10mm x 10mm
   V_max = 10m/s (on the axix of the pipe)
   Re = 13000
   cSmago = 0.13 (for the Smagorinsky dynamics)

   Note :
           En modifiant TMax dans parameters.dat on arrete le prog.
*/

#include "palabos2D.h"
#include "palabos2D.hh"
#include "complexDynamics/smagorinskyDynamics2D.h"

#include <vector>
#include <cmath>
#include <cstdlib>

#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include "io/parallelIO.h"

#include <string>

using namespace plb;
using namespace plb::descriptors;
using namespace std;

typedef double T;
const T pi = (T)4.0*atan(1.0) ;

// Definition du reseau ( D2Q9 )
#define DESCRIPTOR D2Q9Descriptor

// Ecriture du fichier parameters.dat
void writeParameters(plint& iRun, plint& iT, T& TMax, T& logT, T& saveGif, T& saveVTK,
        T& saveProfil, T& initVortiStream, T& saveVortiStream, T& saveCheckpoint,
        T& initImageStream, T& saveImageStream, T& initPointStream, T& savePointStream,
        plint& gifFileIndex, plint& vtkFileIndex, plint& velocityFileIndex, plint& vorticityFileIndex) {
    plb_ofstream outputFile("parameters.dat");
    outputFile << iRun << endl;
    outputFile << iT << endl;
    outputFile << TMax << endl;
    outputFile << logT << endl;
    outputFile << saveGif << endl;
    outputFile << saveVTK << endl;
    outputFile << saveProfil << endl;
    outputFile << saveCheckpoint << endl;
    outputFile << initPointStream << endl;
    outputFile << savePointStream << endl;
    outputFile << initImageStream << endl;
    outputFile << saveImageStream << endl;
    outputFile << initVortiStream << endl;
    outputFile << saveVortiStream << endl;
    outputFile << gifFileIndex << endl;
    outputFile << vtkFileIndex << endl;
    outputFile << velocityFileIndex << endl;
    outputFile << vorticityFileIndex << endl;
}

// Lecture et effacement du fichier parameters.dat
void readParameters(plint& iRun, plint& iTMin, T& TMax, T& logT, T& saveGif, T& saveVTK,
        T& saveProfil, T& initVortiStream, T& saveVortiStream, T& saveCheckpoint,
        T& initImageStream, T& saveImageStream, T& initPointStream, T& savePointStream,
        plint& gifFileIndex, plint& vtkFileIndex, plint& velocityFileIndex, plint& vorticityFileIndex) {
    plb_ifstream outputFile("parameters.dat");
    outputFile >> iRun >> iTMin >> TMax >> logT >> saveGif >> saveVTK >> saveProfil
               >> saveCheckpoint >> initPointStream >> savePointStream >> initImageStream
               >> saveImageStream >> initVortiStream >> saveVortiStream ;
    // Broadcast the input data to all processors;
    global::mpi().bCast(&iRun, 1);
    global::mpi().bCast(&iTMin, 1);
    global::mpi().bCast(&TMax, 1);
    global::mpi().bCast(&logT, 1);
    global::mpi().bCast(&saveGif, 1);
    global::mpi().bCast(&saveVTK, 1);
    global::mpi().bCast(&saveProfil, 1);
    global::mpi().bCast(&saveCheckpoint, 1);
    global::mpi().bCast(&initPointStream, 1);
    global::mpi().bCast(&savePointStream, 1);
    global::mpi().bCast(&initImageStream, 1);
    global::mpi().bCast(&saveImageStream, 1);
    global::mpi().bCast(&initVortiStream, 1);
    global::mpi().bCast(&saveVortiStream, 1);
    global::mpi().bCast(&gifFileIndex, 1);
    global::mpi().bCast(&vtkFileIndex, 1);
    global::mpi().bCast(&velocityFileIndex, 1);
    global::mpi().bCast(&vorticityFileIndex, 1);
}

// Ecoulement turbulent dans un tube a parois lisses ( en 2D )
// Vx = ( y/a )^(1/nn) avec : nn = 7 reference :
// THE VENERABLE 1/17th POWER LAW TURBULENT VELOCITY PROFILE: ...
// APPLIED MATHEMATICS AND COMPUTATION (2005) 163-474. ELSEVIER
T turbulentFlowVelocity(plint composante,plint iX, plint iY, IncomprFlowParam<T> const& parameters,
                        plint xPip, plint yInl, plint yPip) {
    plint nx = parameters.getNx();
    plint ny = parameters.getNy();
    T   uMax = parameters.getLatticeU() ; // Vitesse de l'ecoulement sur l'axe dans la petite section
    T nn = 7;  // Exposent pour le profil turbulent
    T mm = 2;  // Exposent pour le profil de la marche montante
    T xRac = xPip/2.;   // Abscisse du debut du raccord pour la marche montante de l'écoulement
    T x1 = (T)xPip;     // Abscisse aval  du raccordement exponentiel pour la vitesse
    T y1 = (T)yInl;     // Ordonnee amont du raccordement exponentiel pour la vitesse
    T y2 = (T)yPip;     // Ordonnee aval  du raccordement exponentiel pour la vitesse
    T y5 = (T)(ny-1);   // Limite sup. du domaine
    T xx = (T)iX;
    T yy = (T)iY;
    T vx = 0.0;
    T vy = 0.0;
    T fx = y2;
    T fx_prime = 0.0;

    if (xx<xRac){
        fx       = y1;
        fx_prime = 0.0;
    }
    else{
        if (xx<=xPip){
            // On raccorde l'enveloppe de la vitesse par une fonction sinus
			T exposant=pow(((xx-xRac)/(xPip - xRac)),mm);//(xPip-xRac),mm);

            fx = y1 + (y2-y1) * (1.0 + sin(pi*exposant-pi/2.0)) / 2.0;
            // Derivee de la fonction de raccord
            fx_prime = (y2-y1)*mm*pi*pow((xx-xRac)/(xPip-xRac),mm-1)
                     * sin(pi*pow((xx-xRac)/(xPip-xRac),mm)) / (2.0*(xPip-xRac));
        }
        else{
            fx = y2;
            fx_prime = 0.;
        }
    }

    T ymin = fx;
    T ymax = y5-ymin;

    if ((yy>ymin) && (yy<ymax)) {
        T yAxe = y5/2.0;
        T rayon = (ymax-ymin)/2.0;
        T rapSec = (y5-2.0*y2)/(ymax-ymin); // rapSec<=1
        T Yx = (yy>=yAxe) ?  ymax-yy : yy-ymin;
        vx = rapSec * uMax * sin(pi/2.*pow(Yx/rayon,1.0/(nn-3)));
        vy = (yAxe-yy)/rayon * fx_prime * vx;
    }

    T vitesse = (composante==0) ? vx : vy ;
    // T vitesse = (composante==0) ? 1. : 0. ; // On fixe la vitesse a 1 de partout : ça
    return vitesse ;                          // permet de voir le maillage a la 1ere image ZZZZ
}

// Linearly decreasing pressure profile
T turbulentFlowPressure(plint iX, IncomprFlowParam<T> const& parameters) {
    T Lx = parameters.getNx()-1;
    T Ly = parameters.getNy()-1;
    return 8.*parameters.getLatticeNu()*parameters.getLatticeU() / (Ly*Ly) * (Lx/(T)2-(T)iX);
}

// Convert pressure to density according to ideal gas law
T turbulentFlowDensity(plint iX, IncomprFlowParam<T> const& parameters) {
    return turbulentFlowPressure(iX,parameters)*DESCRIPTOR<T>::invCs2 + (T)1;
}

// A functional, used to initialize the velocity for the boundary conditions
template<typename T>
class TurbulentFlowVelocity {
public:
    TurbulentFlowVelocity(IncomprFlowParam<T> parameters_, plint xPip_, plint yInl_, plint yPip_)
        : parameters(parameters_), xPip(xPip_), yInl(yInl_), yPip(yPip_)
    { }
    void operator()(plint iX, plint iY, Array<T,2>& u) const {
        u[0] = turbulentFlowVelocity(0, iX, iY, parameters, xPip, yInl, yPip);
        u[1] = turbulentFlowVelocity(1, iX, iY, parameters, xPip, yInl, yPip);
    }
private:
    IncomprFlowParam<T> parameters;
    plint xPip, yInl, yPip;
};

// A functional, used to initialize a pressure boundary to constant density
template<typename T>
class ConstantDensity {
public:
    ConstantDensity(T density_)
        : density(density_)
    { }
    T operator()(plint iX, plint iY) const {
        return density;
    }
private:
    T density;
};

// A functional, used to create an initial condition for the density and velocity
template<typename T>
class TurbulentFlowVelocityAndDensity {
public:
    TurbulentFlowVelocityAndDensity(IncomprFlowParam<T> parameters_,
	plint xPip_, plint yInl_, plint yPip_)
	: parameters(parameters_), xPip(xPip_), yInl(yInl_), yPip(yPip_)
    { }
    void operator()(plint iX, plint iY, T& rho, Array<T,2>& u) const {
        rho  = turbulentFlowDensity(iX,parameters);
        u[0] = turbulentFlowVelocity(0, iX, iY, parameters, xPip, yInl, yPip);
        u[1] = turbulentFlowVelocity(1, iX, iY, parameters, xPip, yInl, yPip);
    }
private:
    IncomprFlowParam<T> parameters;
    plint xPip, yInl, yPip;
};

// A functional, used to instantiate bounce-back nodes at the locations of the cylinder
template<typename T>
class CylinderShapeDomain2D : public plb::DomainFunctional2D {
public:
    CylinderShapeDomain2D(plb::plint cx_, plb::plint cy_, plb::plint radius)
        : cx(cx_),
          cy(cy_),
          radiusSqr(plb::util::sqr(radius))
    { }
    virtual bool operator() (plb::plint iX, plb::plint iY) const {
        return plb::util::sqr(iX-cx) + plb::util::sqr(iY-cy) <= radiusSqr;
    }
    virtual CylinderShapeDomain2D<T>* clone() const {
        return new CylinderShapeDomain2D<T>(*this);
    }
private:
    plb::plint cx;
    plb::plint cy;
    plb::plint radiusSqr;
};

// A functional, used to instantiate bounce-back nodes at the locations of a rectangle
template<typename T>
class RectangleShapeDomain2D : public plb::DomainFunctional2D {
public:
    RectangleShapeDomain2D(plb::plint x0_, plb::plint x1_, plb::plint y1_, plb::plint y2_)
        : x0(x0_),
          x1(x1_),
          y1(y1_),
          y2(y2_)
    { }
    virtual bool operator() (plb::plint iX, plb::plint iY) const {
        T dif1 = x0-iX;
        T dif2 = x1-iX;
        T dif3 = y1-iY;
        T dif4 = y2-iY;
        return (dif1 < 0)&&(dif2 > 0)&&(dif3 < 0)&&(dif4 > 0);
    }
    virtual RectangleShapeDomain2D<T>* clone() const {
        return new RectangleShapeDomain2D<T>(*this);
    }
private:
    plb::plint x0;
    plb::plint x1;
    plb::plint y1;
    plb::plint y2;
};

// Une fonction qui fabrique 2 corrugations a bords arrondis en vis a vis
// Par la methode du bounce-back
//             |             |    - y4
//             |             |
//             O=============O    - y3
//
//             r1            r2
//             O=============O    - y2
//             |             |
//             |             |    - y1
//             x1            x2

void corrugation(MultiBlockLattice2D<T,DESCRIPTOR>& lattice,plint& x1, plint& x2,
     plint& y1, plint& y2, plint& y3, plint& y4, plint& r1, plint& r2){

    if (r1>0){ // On definit l'arrondi amont de la corrugation
    // En bas
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(x1+r1,y2-r1,r1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(x1,x1+2*r1,y1,y2-r1),//RectangleShapeDomain2D<T>(x1,x1+2*r1,y1,y2-r1),
                   new plb::BounceBack<T,DESCRIPTOR>);
    // En haut
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(x1+r1,y3+r1,r1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(x1,x1+2*r1,y3+r1,y4),
                   new plb::BounceBack<T,DESCRIPTOR>);
    }

    if (r2>0){ // On definit l'arrondi aval de la corrugation
    // En bas
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(x2-r2,y2-r2,r2),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(x2-2*r2,x2,y1,y2-r2),
                   new plb::BounceBack<T,DESCRIPTOR>);
    // En haut
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(x2-r2,y3+r2,r2),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(x2-2*r2,x2,y3+r2,y4),
                   new plb::BounceBack<T,DESCRIPTOR>);
    }

    // On definit la portion droite de la corrugation
    // En bas
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(x1+r1,x2-r2,y1,y2),
                   new plb::BounceBack<T,DESCRIPTOR>);
    // En haut
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(x1+r1,x2-r2,y3,y4),
                   new plb::BounceBack<T,DESCRIPTOR>);
}

// Une fonction qui fabrique 1 peigne qui decroit entre x1, x2, y1, y2 et y3, y4 pour attenuer la pression
//
//         ---------->  Uo
//
//     y2 _ _    _    _    _
//         |X|  |X|  |X|  |X|   _    _    _
//         |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|   _    _    _    _
//         |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|
//         |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|
//     y1 _|X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|  |X|
//     ==================================================================
//       Le bas du silencieux doit etre fermé par la frontiere du domaine
//
//       lF = largeur des lames de fluide ;lM = largeur des murs

void multiCorrug(MultiBlockLattice2D<T,DESCRIPTOR>& lattice,plint& x1, plint& x2,
    plint& y1, plint& y2, plint& y3, plint& y4, plint& lM, plint& lF){
    plint r0  = 0;
    plint xc1 = 0;
    plint xc2 = 0;
    plint yc1 = y1+lF;;
    plint yc4 = y4-lF;
    T alpha   = 1.0; // La derniere corrugation fait alpha fois la Hauteur de la premiere corrugation
    plint yc2,yc3;
    T     x01,y01;
    if (alpha<0.0 || alpha>1.0){
        pcout << "ERROR in function multiCorrug : alpha must be between 0 and 1" << endl;
        exit(1);
    }
    for (plint ii=1; ii<=(x2-x1-lF)/(lM+lF); ii++) {
        xc1 = x1+lF+(ii-1)*(lM+lF);
        xc2 = xc1+lM;
        x01 = ((T)(xc1-x1))/((T)(x2-x1));
        y01 = (1.0-x01)*exp(x01);
        yc2 = lF +      (y2   -lF)*(alpha+(1.0-alpha)*y01);
        yc3 = y4 - lF + (y3-y4+lF)*(alpha+(1.0-alpha)*y01);
        corrugation(lattice, xc1, xc2, yc1, yc2, yc3, yc4, r0, r0);
    }
}

// Definition d'une fonction qui fabrique une corrugation avec un silencieux amont
//
//         ---------->  Uo
//
//         o=================o - y2
//         |XXXXXXXXXXXXXX||X|
//         ================|X|
//                         |X|
//         =============== |X|
//         |XXXXXXXXXXXXX| |X|
//         =============== |X|
//                         |X|
//         =============== |X|
//         |XXXXXXXXXXXXX| |X|
//         =============== |X|
//                         |X| - y1
//
//       Le bas du silencieux doit etre fermé par la frontiere du domaine
//
//       lF = largeur des lames de fluide ;lM = largeur des murs

void corrugSilUpwd(MultiBlockLattice2D<T,DESCRIPTOR>& lattice,plint& x1, plint& x2, plint& y1,
     plint& y2, plint& y3, plint& y4, plint& lM, plint& lF, plint& lTop, plint& r1, plint& r2){

    plint r0 = 0;
    plint rM = (r1>r2) ? r1 : r2;
    plint lS = (2*rM>lTop) ? 2*rM : lTop;

//  Sommet de la corrugation
    plint yc1 = y2 - lS;
    plint yc4 = y3 + lS;
    corrugation(lattice, x1, x2, yc1, y2, y3, yc4, r1, r2);

//  Mur avec la cavite
    plint xc1 = x2 - lS;
    plint yc2 = y2 - lS + 1;
    plint yc3 = y3 + lS - 1;
    corrugation(lattice, xc1, x2, y1, yc2, yc3, y4,  r0, r0);

//  Les murs interieurs
    plint xc2 = x2 - lS - lF;
    for (plint ii=1; ii<=(y2-y1-lS-lF)/(lM+lF); ii++) {
        yc2 = y2  - lS - lF - (ii-1)*(lM+lF);
        yc1 = yc2 - lM;
        yc3 = y3  + lS + lF + (ii-1)*(lM+lF);
        yc4 = yc3 + lM;
        corrugation(lattice, x1, xc2, yc1, yc2, yc3, yc4, r0, r0);
    }
}

// Definition de la geometrie et des conditions aux limites de l'ecoulement
void boundaryConditionsSetup( MultiBlockLattice2D<T,DESCRIPTOR>& lattice,
                              IncomprFlowParam<T> const& parameters,
                              OnLatticeBoundaryCondition2D<T,DESCRIPTOR>& boundaryCondition,
                              plint Xb0, plint Xb1, plint Xb2, plint Xb3, plint Xb4,
                              plint Y0, plint Y1, plint Y2, plint Y3, plint Y4, plint Y5,
                              plint xPip, plint yInl, plint yPip, plint nCav, plint xCav, plint lCav,
                              plint lCor, plint xSil, plint rCorru, plint rInlet)
{
    const plint nx = parameters.getNx();
    const plint ny = parameters.getNy();

    plint ncav = nCav; // Nombre de cavites

    plint xpip = xPip; // Bord amont de la cavite
    plint xcav = xCav; // Bord amont de la cavite
    plint lcav = lCav; // Longueur de la cavite
    plint lcor = lCor; // Longueur de la corrugation
    plint xsil = xSil; // Abscisse du second silencieux

    plint r0 = 0;      // Arrondi en sortie du tuyau
    plint r1 = rInlet; // Arrondi de l'entree d'air
    plint r2 = rCorru; // Arrondi de la corrugation

    plint lS = 20; // lS = hauteur du sommet de la corrugation (lS >= 2*r1 et lS >= 2*r2)
    plint lM =  4; // lM = largeur du mur du silencieux
    plint lF =  8; // lF = largeur de la lame de fluide dans le silencieux

    plint xMin = 0;
    plint xMax = nx-1;

    plint xb0 = Xb0;
    plint xb1 = Xb1;
    plint xb2 = Xb2;
    plint xb3 = Xb3;
    plint xb4 = Xb4;

    plint y0 = Y0;
    plint y1 = Y1;
    plint y2 = Y2;
    plint y3 = Y3;
    plint y4 = Y4;
    plint y5 = Y5;

    plint xc1 = 0;
    plint xc2 = 0;
    plint yc0 = 0;
    plint yc5 = 0;

    // Les bornes du domaine
    Box2D  infInlet(xMin, xMin, y0+1, y1);
    Box2D     inlet(xMin, xMin, y1+1, y4-1);
    Box2D  supInlet(xMin, xMin, y4, y5-1);

    Box2D bottomWall1(xb0, xb3-1, y0, y0);
    Box2D    topWall1(xb0, xb3-1, y5, y5);

    Box2D bottomWall2(xb3, xb4, y2, y2);//Box2D bottomWall2(xb3, xb4, y2, y2);
    Box2D    topWall2(xb3, xb4, y3, y3);

    Box2D      outlet(xMax, xMax, y2+1, y3-1);

    // Create Velocity boundary conditions everywhere
    boundaryCondition.setVelocityConditionOnBlockBoundaries ( lattice, inlet);
    boundaryCondition.setVelocityConditionOnBlockBoundaries ( lattice, infInlet);
    boundaryCondition.setVelocityConditionOnBlockBoundaries ( lattice, supInlet);
    boundaryCondition.setVelocityConditionOnBlockBoundaries ( lattice, bottomWall1);
    boundaryCondition.setVelocityConditionOnBlockBoundaries ( lattice, topWall1);
    boundaryCondition.setVelocityConditionOnBlockBoundaries ( lattice, bottomWall2);
    boundaryCondition.setVelocityConditionOnBlockBoundaries ( lattice, topWall2);

    // .. except on right boundary, where we prefer an outflow condition
    //    (zero velocity-gradient).
    boundaryCondition.setVelocityConditionOnBlockBoundaries ( lattice, outlet, boundary::outflow );

    // La condition de vitesse aux limites en vitesse est donnees par TurbulentFlowVelocity
    setBoundaryVelocity ( lattice, lattice.getBoundingBox(),
            TurbulentFlowVelocity<T>(parameters, xPip, yInl, yPip) );

    // On force la vitesse a zero sur les murs
    setBoundaryVelocity(lattice,    infInlet, Array<T,2>(0.,0.) );
    setBoundaryVelocity(lattice,    supInlet, Array<T,2>(0.,0.) );
    setBoundaryVelocity(lattice, bottomWall1, Array<T,2>(0.,0.) );
    setBoundaryVelocity(lattice,    topWall1, Array<T,2>(0.,0.) );
    setBoundaryVelocity(lattice, bottomWall2, Array<T,2>(0.,0.) );
    setBoundaryVelocity(lattice,    topWall2, Array<T,2>(0.,0.) );

    setBoundaryDensity ( lattice, outlet, ConstantDensity<T>(1.) );

    initializeAtEquilibrium ( lattice, lattice.getBoundingBox(),
            TurbulentFlowVelocityAndDensity<T>(parameters, xPip, yInl, yPip) );

    // construction de 2 corrugations a l'amont de la cavite
    xc1 = xpip;
    xc2 = xcav;
    yc0 = y0+1;
    yc5 = y5-1;
    corrugSilUpwd(lattice, xc1, xc2, yc0, y2, y3, yc5, lM, lF, lS, r1, r2); // Corrugation 1 : inlet

    // construction de 2 corrugations a l'aval de la cavite
    xc1 = xcav+lcav;
    xc2 = xb3-1;
    yc0 = y0+1;
    yc5 = y5-1;//yc5 = y5-1;
    corrugation(lattice, xc1, xc2, yc0, y2, y3, yc5, r2, r0); // Corrugation 2 : sortie

    // construction du "silencieux" a l'entree
    xc1 = xMin;
    xc2 = xpip;
    yc0 = y0+1;
    yc5 = y5-1;
    multiCorrug(lattice, xc1, xc2, yc0, y1, y4, yc5, lM, lF);

    lattice.initialize();
}

// Ecriture des images GIF
void writeGif(MultiBlockLattice2D<T,DESCRIPTOR>& lattice, plint iter)
{
    const plint imSize = 800;
    ImageWriter<T> imageWriter("leeloo");
    imageWriter.writeScaledGif(createFileName("u", iter, 6), *computeVelocityNorm(lattice), imSize, imSize );
}

// Ecriture des images VTK
void writeVTK(MultiBlockLattice2D<T,DESCRIPTOR>& lattice, IncomprFlowParam<T> const& parameters, plint iter)
{
    const T dx = parameters.getDeltaX();
    const T dt = parameters.getDeltaT();
    VtkImageOutput2D<T> vtkOut(createFileName("vtk", iter, 6), dx);
    vtkOut.writeData<float>(*computeVelocityNorm(lattice), "velocityNorm", dx/dt);
    vtkOut.writeData<2,float>(*computeVelocity(lattice), "velocity", dx/dt);
}

// Ecriture des images de vorticite
void writeVor(MultiBlockLattice2D<T,DESCRIPTOR>& lattice, IncomprFlowParam<T> const& parameters,
Box2D domaineVorti, plint iter)
{
    const T dx = parameters.getDeltaX();
    const T dt = parameters.getDeltaT();
    // Box2D domaineVorti(boxVortx1,boxVortx2,boxVorty1,boxVorty2);
    VtkImageOutput2D<T> vortiOut(createFileName("v", iter, 6), dx);
    vortiOut.writeData<float>(*computeVorticity(*computeVelocity(lattice),domaineVorti), "vorticity", dx/dt);
}

/* Definition de la geometrie

+===========+        +========+                          +===========+ -- y5=(ny-1)
|||||||||||||        |        |                          |||||||||||||
|||||||||||||        |        |        +=================||||||||||||| -- y4 = y5-y1
            |        |        |        |
            o========o        o========o-----------------------------+ -- y3 = y5-y2



----->
  U
               lInl      lCav     lCor
            <--------><-------><-------->
            o========o         o========o-----------------------------+ -- y2 = yPip
            |        |         |        |
|||||||||||||        |         |        +=================||||||||||||| -- y1 = yPip
|||||||||||||        |         |        :                 |||||||||||||
+===========+        +=========+        :                 +===========+ -- y0=0
|           |        |         |        |                             |
x0=0      xPip     xCav       xCav     xCav
\__________/                 +lCav    +lCav               \__________/
Inlet Silencer       \_________/      +lCor              Oullet Silencer
                     Cavity
*/
int main(int argc, char **argv, bool verbous) {
    plbInit(&argc, &argv, &verbous);

    IncomprFlowParam<T> parameters(
            (T) 0.017, // u_LB : c=1/sqrt(3) => u_LB = u_SI/sqrt(3)/340 :0.017 -> 10 m/s
            (T) 6500.,// Re = (u_0 * d) / nu , avec d=1 cm
            600,    // N : Nombre de points : On maille de 0 a N-1 :
            4.,    // lx                        _____________          ____________
            1.    // ly
            );

    const plint nx = parameters.getNx();
    const plint ny = parameters.getNy();

    const plint rCorru = 10; // Arrondi des bords de cavite
    const plint rInlet = 10; // Arrondi de l'entree d'air

    const plint nCav =   1;  //   Le nombre de cavites

    // Dimensions en unite physiques (mm)
    const plint h_Dom = 30;           //  hauteur totale totale du maillage
    const plint x_Pip = 40;           //  abscisse de l'entree du tube = longueur du premier silencieux
    const plint y_Inl =  5;           //  Ordonnee du premier tube (inlet )
    const plint y_Pip = 10;           //  Ordonnee du second tube ( riser )
    const plint l_Inl = 10;           //  longueur de tube en amont de la cavite
    const plint l_Cav = 10;           //  longueur de la cavite
    const plint l_Cor =  0;           //  longueur de la corrugation (s'il y a plus d'une cavite)
    const plint x_Sil = 70;           //  abscisse du second silencieux

    const plint l_Dom = h_Dom*(nx-1)/(ny-1);   //  Longueur totale du maillage
    const plint l_Pip = l_Dom - x_Pip;        //  Longueur totale du tube
    const plint x_Cav = l_Inl + x_Pip;       //  Abscisse de la cavite

    // Dimensions en unite maillage (ENTIERS)
    const plint xPip = (plint)(0.5+x_Pip*(ny-1)/h_Dom);
    const plint yInl = (plint)(0.5+y_Inl*(ny-1)/h_Dom);
    const plint yPip = (plint)(0.5+y_Pip*(ny-1)/h_Dom);
    const plint xCav = (plint)(0.5+x_Cav*(ny-1)/h_Dom);
    const plint lCav = (plint)(0.5+l_Cav*(ny-1)/h_Dom);
    const plint lCor = (plint)(0.5+l_Cor*(ny-1)/h_Dom);
    const plint xSil = (plint)(0.5+x_Sil*(ny-1)/h_Dom);

    // Definition des abscisses des SparceBlocks
    const plint xb0 = 0;
    const plint xb2 = (xPip+2*rInlet);
    const plint xb1 = (xb0+xb2)/2;
    const plint xb3 = xCav+lCav+2*rCorru;
    const plint xb4 = nx-1;

    // Definition des ordonnees des SparceBlocks
    const plint y0 = 0;
    const plint y5 = ny-1;
    const plint y1 = yInl;
    const plint y2 = yPip;
    const plint y3 = y5-y2;
    const plint y4 = y5-y1;

    const plint nbpX = 100; // Pour la sortie de fieldPoint1_Vx
    const plint nbpY =  50; // Pour la sortie de fieldPoint2_Vx et fieldPoint3_Vx

    const T dx = parameters.getDeltaX();
    const T dt = parameters.getDeltaT();

    plint entier, xData, yData, iT, iRun, iTMin;
    plint gifFileIndex, vtkFileIndex, velocityFileIndex, vorticityFileIndex;

    T reel, TMax, logT, saveGif, saveVTK, saveProfil, initVortiStream, saveVortiStream,
      saveCheckpoint, initImageStream, saveImageStream, initPointStream, savePointStream;

    // Noms dynamiques des fichiers de sortie
    char vorticityFileName[27];
    char xVelocityFileName[27];
    char yVelocityFileName[27];

    // Box definition for writing xVelocity and yVelocity
    const plint boxVx1 = xPip + 1;
    const plint boxVx2 = xCav + 2*lCav;
    const plint boxVy1 = y0 + 1;
    const plint boxVy2 = (y0+y5)/2;

    // Box definition for writing the vorticity
    const plint boxVortx1 = xPip + 1;
    const plint boxVortx2 = xCav + 2*lCav;
    const plint boxVorty1 = y0 + 1;
    const plint boxVorty2 = (y0+y5)/2;

    plb_ifstream outputFile("parameters.dat");
    if (outputFile.is_open()){
        readParameters ( iRun, iTMin, TMax, logT, saveGif, saveVTK, saveProfil, initVortiStream,
        saveVortiStream, saveCheckpoint, initImageStream, saveImageStream, initPointStream,
        savePointStream, gifFileIndex, vtkFileIndex, velocityFileIndex, vorticityFileIndex );
    }
    else {
        iRun               = (plint)1 ; // numero de run (1<=iRun<9)
        iTMin              = (plint)0 ; // indice iT du debut du calcul
        TMax               = (T) 80.1 ; // temps max en secondes pour la simulation
        logT               = (T)  0.01; // step d'ecriture du fichier log
        saveGif            = (T)  0.05; // step d'ecriture des images gif (=0 => pas de sortie gif)
        saveVTK            = (T)  0   ; // step d'ecriture des fichiers vtk (=0 => aucune sortie)
        saveProfil         = (T)  0.01; // step d'ecriture des profils de vitesses (=0 => aucune sortie)
        saveCheckpoint     = (T) 10.0 ; // step d'ecriture du fichier checkPoint (=0 => aucune sauvegarde)
        initPointStream    = (T)  0.0 ; // date du debut d'ecriture des vitesses/densites en 1D
        savePointStream    = (T)  0.01; // step d'ecriture des vitesses en 1D (=0 => aucun streaming 1D)
        initImageStream    = (T)  0.0 ; // date du debut d'ecriture des vitesses en 2D
        saveImageStream    = (T)  0.05; // step d'ecriture des vitesses en 2D (=0 => aucun streaming 2D)
        initVortiStream    = (T)  0.0 ; // debut d'ecriture de la Vorticite
        saveVortiStream    = (T)  0.05; // step d'ecriture de la Vorticite (=0 => aucune sortie)
        gifFileIndex       = (plint) 0;
        vtkFileIndex       = (plint) 0;
        velocityFileIndex  = (plint) 0;
        vorticityFileIndex = (plint) 0;
    }

    writeParameters ( iRun, iTMin, TMax, logT, saveGif, saveVTK, saveProfil, initVortiStream,
        saveVortiStream, saveCheckpoint, initImageStream, saveImageStream, initPointStream,
        savePointStream, gifFileIndex, vtkFileIndex, velocityFileIndex, vorticityFileIndex );

    // Repertoire de sortie pour les images
    global::directories().setOutputDir("./tmp/gif/");

    plb_ofstream geometry("./tmp/geometry.txt",ios::app);
    geometry << "l_Domain = " << "," << l_Dom << endl;
    geometry << "l_Pipe   = " << "," << l_Pip << endl;
    geometry << "Diameter = " << "," << h_Dom-2*y_Pip << endl;
    geometry << "l_Inlet  = " << "," << l_Inl << endl;
    geometry << "l_Cavity = " << "," << l_Cav << endl;
    geometry << "Nx       = " << "," << nx << endl;
    geometry << "Ny       = " << "," << ny << endl;
    geometry << "dx       = " << "," << dx << endl;
    geometry << "dt       = " << "," << dt << endl;
    geometry << "Nx_V2D   = " << "," << boxVx2-boxVx1+1 << endl;
    geometry << "Ny_V2D   = " << "," << boxVy2-boxVy1+1 << endl;
    geometry << "Nx_Vort  = " << "," << boxVortx2-boxVortx1+1 << endl;
    geometry << "Ny_Vort  = " << "," << boxVorty2-boxVorty1+1 << endl;
    geometry << "T0_V1D   = " << "," << initPointStream << endl;
    geometry << "dT_V1D   = " << "," << savePointStream << endl;
    geometry << "T0_V2D   = " << "," << initImageStream << endl;
    geometry << "dT_V2D   = " << "," << saveImageStream << endl;
    geometry << "T0_Vort  = " << "," << initVortiStream << endl;
    geometry << "dT_Vort  = " << "," << saveVortiStream << endl;

    plb_ofstream profilVx1("./tmp/profilVx1.dat",ios::app);
    plb_ofstream profilVy1("./tmp/profilVy1.dat",ios::app);
    plb_ofstream profilVx2("./tmp/profilVx2.dat",ios::app);
    plb_ofstream profilVy2("./tmp/profilVy2.dat",ios::app);

    plb_ofstream averageDensity("./tmp/averageDensity.dat",ios::app);
    plb_ofstream averageEnergy ( "./tmp/averageEnergy.dat",ios::app);

    plb_ofstream fieldPointDate("./tmp/fieldPointDate.dat",ios::app);

    plb_ofstream fieldPoint1_Vx("./tmp/fieldPoint1_Vx.dat",ios::app);
    plb_ofstream fieldPoint1Rho("./tmp/fieldPoint1Rho.dat",ios::app);

    plb_ofstream fieldPoint2_Vx("./tmp/fieldPoint2_Vx.dat",ios::app);
    plb_ofstream fieldPoint2_Vy("./tmp/fieldPoint2_Vy.dat",ios::app);
    plb_ofstream fieldPoint2Rho("./tmp/fieldPoint2Rho.dat",ios::app);

    plb_ofstream fieldPoint3_Vx("./tmp/fieldPoint3_Vx.dat",ios::app);
    plb_ofstream fieldPoint3_Vy("./tmp/fieldPoint3_Vy.dat",ios::app);
    plb_ofstream fieldPoint3Rho("./tmp/fieldPoint3Rho.dat",ios::app);

    plb_ofstream fieldPoint4_Vx("./tmp/fieldPoint4_Vx.dat",ios::app);
    plb_ofstream fieldPoint4_Vy("./tmp/fieldPoint4_Vy.dat",ios::app);
    plb_ofstream fieldPoint4Rho("./tmp/fieldPoint4Rho.dat",ios::app);

    //Output Density
	 plb_ofstream xAxisVxProfiles1("tmp/xAxisVxProfiles1.dat");
    plb_ofstream xAxisVxProfiles2("tmp/xAxisVxProfiles2.dat");
    plb_ofstream xAxisVxProfiles3("tmp/xAxisVxProfiles3.dat");
    plb_ofstream xAxisVxProfiles4("tmp/xAxisVxProfiles4.dat");
    plb_ofstream xAxisVxProfiles5("tmp/xAxisVxProfiles5.dat");
    plb_ofstream xAxisVxProfiles6("tmp/xAxisVxProfiles6.dat");
    plb_ofstream xAxisVxProfiles7("tmp/xAxisVxProfiles7.dat");
    plb_ofstream xAxisVxProfiles8("tmp/xAxisVxProfiles8.dat");
    plb_ofstream xAxisVxProfiles9("tmp/xAxisVxProfiles9.dat");
    plb_ofstream xAxisVxProfiles10("tmp/xAxisVxProfiles10.dat");
    plb_ofstream xAxisVxProfiles11("tmp/xAxisVxProfiles11.dat");
    plb_ofstream xAxisVxProfiles12("tmp/xAxisVxProfiles12.dat");
    plb_ofstream xAxisVxProfiles13("tmp/xAxisVxProfiles13.dat");
    plb_ofstream xAxisVxProfiles14("tmp/xAxisVxProfiles14.dat");
    plb_ofstream xAxisVxProfiles15("tmp/xAxisVxProfiles15.dat");
    plb_ofstream xAxisVxProfiles16("tmp/xAxisVxProfiles16.dat");
    plb_ofstream xAxisVxProfiles17("tmp/xAxisVxProfiles17.dat");
    plb_ofstream xAxisVxProfiles18("tmp/xAxisVxProfiles18.dat");
    plb_ofstream xAxisVxProfiles19("tmp/xAxisVxProfiles19.dat");
    plb_ofstream xAxisVxProfiles20("tmp/xAxisVxProfiles20.dat");
    plb_ofstream xAxisVxProfiles21("tmp/xAxisVxProfiles21.dat");
    plb_ofstream xAxisVxProfiles22("tmp/xAxisVxProfiles22.dat");
    plb_ofstream xAxisVxProfiles23("tmp/xAxisVxProfiles23.dat");
    plb_ofstream xAxisVxProfiles24("tmp/xAxisVxProfiles24.dat");
    plb_ofstream xAxisVxProfiles25("tmp/xAxisVxProfiles25.dat");
    plb_ofstream xAxisVxProfiles26("tmp/xAxisVxProfiles26.dat");
    plb_ofstream xAxisVxProfiles27("tmp/xAxisVxProfiles27.dat");
    plb_ofstream xAxisVxProfiles28("tmp/xAxisVxProfiles28.dat");
    plb_ofstream xAxisVxProfiles29("tmp/xAxisVxProfiles29.dat");
    plb_ofstream xAxisVxProfiles30("tmp/xAxisVxProfiles30.dat");
    plb_ofstream xAxisVxProfiles31("tmp/xAxisVxProfiles31.dat");
    plb_ofstream xAxisVxProfiles32("tmp/xAxisVxProfiles32.dat");
    plb_ofstream xAxisVxProfiles33("tmp/xAxisVxProfiles33.dat");
    plb_ofstream xAxisVxProfiles34("tmp/xAxisVxProfiles34.dat");
    plb_ofstream xAxisVxProfiles35("tmp/xAxisVxProfiles35.dat");
    plb_ofstream xAxisVxProfiles36("tmp/xAxisVxProfiles36.dat");
    plb_ofstream xAxisVxProfiles37("tmp/xAxisVxProfiles37.dat");
    plb_ofstream xAxisVxProfiles38("tmp/xAxisVxProfiles38.dat");
    plb_ofstream xAxisVxProfiles39("tmp/xAxisVxProfiles39.dat");
    plb_ofstream xAxisVxProfiles40("tmp/xAxisVxProfiles40.dat");
    plb_ofstream xAxisVxProfiles41("tmp/xAxisVxProfiles41.dat");
    plb_ofstream xAxisVxProfiles42("tmp/xAxisVxProfiles42.dat");
    plb_ofstream xAxisVxProfiles43("tmp/xAxisVxProfiles43.dat");
    plb_ofstream xAxisVxProfiles44("tmp/xAxisVxProfiles44.dat");
    plb_ofstream xAxisVxProfiles45("tmp/xAxisVxProfiles45.dat");
    plb_ofstream xAxisVxProfiles46("tmp/xAxisVxProfiles46.dat");
    plb_ofstream xAxisVxProfiles47("tmp/xAxisVxProfiles47.dat");
    plb_ofstream xAxisVxProfiles48("tmp/xAxisVxProfiles48.dat");
    plb_ofstream xAxisVxProfiles49("tmp/xAxisVxProfiles49.dat");
    plb_ofstream xAxisVxProfiles50("tmp/xAxisVxProfiles50.dat");
    plb_ofstream xAxisVxProfiles51("tmp/xAxisVxProfiles51.dat");


    // Compute only the x-direction of the velocity (direction of the flow).
    plint const xComponent = 0;

    // Compute only the y-direction of the velocity (normal to the direction of the flow).
    plint const yComponent = 1;

    // Create a block distribution with the three added blocks.
    plint envelopeWidth = 1;
    SparseBlockStructure2D sparseBlock(nx, ny);
    sparseBlock.addBlock(Box2D(xb0, xb1-1, y0, y5), sparseBlock.nextIncrementalId());
    sparseBlock.addBlock(Box2D(xb1, xb2-1, y0, y5), sparseBlock.nextIncrementalId());
    sparseBlock.addBlock(Box2D(xb2, xb3-1, (y2)*3/4, (y5-(y2*3/4))), sparseBlock.nextIncrementalId());//sparseBlock.addBlock(Box2D(xb2, xb3-1, y0, y5), sparseBlock.nextIncrementalId());
    sparseBlock.addBlock(Box2D(xb3, xb4,   y2, y3), sparseBlock.nextIncrementalId());

    /* // Instantiate the multi-block, based on the created block distribution on default parameters.
    // with BGK dynamics
    writeLogFile(parameters, "BGK dynamics");
    MultiBlockLattice2D<T, DESCRIPTOR> lattice (
        MultiBlockManagement2D ( sparseBlock, defaultMultiBlockPolicy2D().getThreadAttribution(),
            envelopeWidth ),
        defaultMultiBlockPolicy2D().getBlockCommunicator(),
        defaultMultiBlockPolicy2D().getCombinedStatistics(),
        defaultMultiBlockPolicy2D().getMultiCellAccess<T,DESCRIPTOR>(),
        new BGKdynamics<T,DESCRIPTOR>(parameters.getOmega())
    );
    */ // End instantiate BGK dynamics.

    /* // Instantiate the multi-block, based on the created block distribution on default parameters.
    // with Regularized BGK dynamics
    writeLogFile(parameters, "Regularized BGK dynamics");
    MultiBlockLattice2D<T, DESCRIPTOR> lattice (
        MultiBlockManagement2D ( sparseBlock, defaultMultiBlockPolicy2D().getThreadAttribution(),
            envelopeWidth ),
        defaultMultiBlockPolicy2D().getBlockCommunicator(),
        defaultMultiBlockPolicy2D().getCombinedStatistics(),
        defaultMultiBlockPolicy2D().getMultiCellAccess<T,DESCRIPTOR>(),
        new RegularizedBGKdynamics<T,DESCRIPTOR>(parameters.getOmega())
    );
    */ // End instantiate Regularized BGK dynamics.

    // Instantiate the multi-block, based on the created block distribution on default parameters.
    // with SMAGORINSKY dynamics
    T cSmago = 0.12;
    writeLogFile(parameters, "Smagorinsky dynamics cSmago=0.12");
    MultiBlockLattice2D<T, DESCRIPTOR> lattice (
        MultiBlockManagement2D ( sparseBlock, defaultMultiBlockPolicy2D().getThreadAttribution(),
            envelopeWidth ),
        defaultMultiBlockPolicy2D().getBlockCommunicator(),
        defaultMultiBlockPolicy2D().getCombinedStatistics(),
        defaultMultiBlockPolicy2D().getMultiCellAccess<T,DESCRIPTOR>(),
        new SmagorinskyBGKdynamics<T,DESCRIPTOR>(parameters.getOmega(), cSmago)
    );
    // End instantiate Smagorinsky dynamics.

    // Display informations on the sizes of the sparseBlocks
    geometry << getMultiBlockInfo(lattice) << endl;

    OnLatticeBoundaryCondition2D<T,DESCRIPTOR>* boundaryCondition
        = createLocalBoundaryCondition2D<T,DESCRIPTOR>();

    boundaryConditionsSetup(lattice, parameters, *boundaryCondition, xb0, xb1, xb2, xb3, xb4,
        y0, y1, y2, y3, y4, y5, xPip, yInl, yPip, nCav, xCav, lCav, lCor, xSil, rCorru, rInlet);

    // Load saved data from a previous simulation, if the initial time step is larger than zero.
    if (iTMin>0) {
        pcout << "Reading the state of the simulation in file chechpoint.dat" << endl;
        loadBinaryBlock(lattice, "checkpoint.dat");
    }

    // Main loop over time iterations.
    for (iT=iTMin; iT*dt<=TMax; ++iT) {
        // At this point, the state of the lattice corresponds to the
        // discrete time iT. However, the stored averages (getStoredAverageEnergy
        // and getStoredAverageDensity) correspond to the previous time iT-1.

        if (saveGif>0) {
            if (iT%parameters.nStep(saveGif)==0) {
            pcout << "Saving Gif ... at t =  " << iT*dt << endl;
            writeGif(lattice, gifFileIndex++);
            }
        }

        if (saveVTK>0) {
            if (iT%parameters.nStep(saveVTK)==0) {
            pcout << "Saving VTK ... at t =  " << iT*dt << endl;
            writeVTK(lattice, parameters, vtkFileIndex++);
            }
        }

        // Ecriture des vitesses et des densites sur l'axe du tube
        if ( (savePointStream>0) && ((iT*dt)>=initPointStream) ) {
            if (iT%parameters.nStep(savePointStream)==0) {
                fieldPointDate << (iT*dt) << endl;
                averageDensity << setprecision(10) << getStoredAverageDensity<T>(lattice) << endl;
                averageEnergy  << setprecision(10) <<  getStoredAverageEnergy<T>(lattice) << endl;
                for (plint ix=1; ix<=nbpX; ix++) {
                //  xData = xPip+(plint)(0.5+((nx-1-xPip)/(nbpX-1)*(ix-1))); // Micro 1 a x=xPip
                    xData = (plint)(0.5+((nx-1)/(nbpX-1)*(ix-1)));          // Micro 1 a x=0
                    yData = (plint)(0.4*(ny-1)); // Un peu sous l'axe (pour voir le 1er mode transverse)
                    Box2D measurePoint(xData, xData, yData, yData);
                    fieldPoint1_Vx << setprecision(10) <<  *multiply ( dx/dt,
                            *computeVelocityComponent (lattice, measurePoint, xComponent) );
                    fieldPoint1Rho << setprecision(10) <<  *computeDensity (lattice, measurePoint);
                }
                fieldPoint1_Vx << endl;
                fieldPoint1Rho << endl;
            }
        }

        // Ecriture des vitesses et des densites dans la section de l'entree d'air
        if ( (savePointStream>0) && ((iT*dt)>=initPointStream) ) {
            if (iT%parameters.nStep(savePointStream)==0) {
                for (plint iy=1; iy<=nbpY; iy++) {
                    xData = xPip;
                    yData = y2+1+(plint)(0.5+(y3-y2-2)*(iy-1)/(nbpY-1));
                    Box2D measurePoint(xData, xData, yData, yData);
                    fieldPoint2_Vx << setprecision(10) <<  *multiply ( dx/dt,
                            *computeVelocityComponent (lattice, measurePoint, xComponent) );
                    fieldPoint2_Vy << setprecision(10) <<  *multiply ( dx/dt,
                            *computeVelocityComponent (lattice, measurePoint, yComponent) );
                    fieldPoint2Rho << setprecision(10) <<  *computeDensity (lattice, measurePoint);
                }
                fieldPoint2_Vx << endl;
                fieldPoint2_Vy << endl;
                fieldPoint2Rho << endl;
            }
        }

        // Ecriture des vitesses et des densites dans la section a l'entree de la cavite
        if ( (savePointStream>0) && ((iT*dt)>=initPointStream) ) {
            if (iT%parameters.nStep(savePointStream)==0) {
                for (plint iy=1; iy<=nbpY; iy++) {
                    xData = xCav;
                    yData = y2+1+(plint)(0.5+(y3-y2-2)*(iy-1)/(nbpY-1));
                    Box2D measurePoint(xData, xData, yData, yData);
                    fieldPoint3_Vx << setprecision(10) <<  *multiply ( dx/dt,
                            *computeVelocityComponent (lattice, measurePoint, xComponent) );
                    fieldPoint3_Vy << setprecision(10) <<  *multiply ( dx/dt,
                            *computeVelocityComponent (lattice, measurePoint, yComponent) );
                    fieldPoint3Rho << setprecision(10) <<  *computeDensity (lattice, measurePoint);
                }
                fieldPoint3_Vx << endl;
                fieldPoint3_Vy << endl;
                fieldPoint3Rho << endl;
            }
        }

        // Ecriture des vitesses et des densites dans la section a l'aval de la cavite
        if ( (savePointStream>0) && ((iT*dt)>=initPointStream) ) {
            if (iT%parameters.nStep(savePointStream)==0) {
                for (plint iy=1; iy<=nbpY; iy++) {
                    xData = xCav+lCav;
                    yData = y2+1+(plint)(0.5+(y3-y2-2)*(iy-1)/(nbpY-1));
                    Box2D measurePoint(xData, xData, yData, yData);
                    fieldPoint4_Vx << setprecision(10) <<  *multiply ( dx/dt,
                            *computeVelocityComponent (lattice, measurePoint, xComponent) );
                    fieldPoint4_Vy << setprecision(10) <<  *multiply ( dx/dt,
                            *computeVelocityComponent (lattice, measurePoint, yComponent) );
                    fieldPoint4Rho << setprecision(10) <<  *computeDensity (lattice, measurePoint);
                }
                fieldPoint4_Vx << endl;
                fieldPoint4_Vy << endl;
                fieldPoint4Rho << endl;
            }
        }

        // Ecriture des vitesses en 2D
        if ( (saveImageStream>0) && (iT*dt>=initImageStream) ) {
            if (iT%parameters.nStep(saveImageStream)==0) {
                // Output file names
                sprintf(xVelocityFileName, "./tmp/velocity/vx%06d.dat", (int) velocityFileIndex);
                plb_ofstream xVelocity(xVelocityFileName,ios::app);
                sprintf(yVelocityFileName, "./tmp/velocity/vy%06d.dat", (int) velocityFileIndex);
                plb_ofstream yVelocity(yVelocityFileName,ios::app);
                // Definition du domaine du trace
                Box2D domaine2d(boxVx1, boxVx2, boxVy1, boxVy2);
                //trace Vx
                xVelocity
                    << setprecision(10)
                    << *multiply ( dx/dt, *computeVelocityComponent (lattice, domaine2d, xComponent) )
                    << endl;
                //trace Vx
                yVelocity
                    << setprecision(10)
                    << *multiply ( dx/dt, *computeVelocityComponent (lattice, domaine2d, yComponent) )
                    << endl;
                // Incrementing velicity file name
                velocityFileIndex ++;
            }
        }

        // Writing the Vorticity
        if ( (saveVortiStream>0) && (iT*dt>=initVortiStream) ) {
            if (iT%parameters.nStep(saveVortiStream)==0) {
                // Output file name
                sprintf(vorticityFileName, "./tmp/vorticity/v%06d.dat", (int) vorticityFileIndex);
                plb_ofstream vorticity(vorticityFileName,ios::app);
                // Definition du domaine du trace
                Box2D domaine2d(boxVortx1,boxVortx2,boxVorty1,boxVorty2);
                //trace la vorticite
                vorticity
                    << setprecision(10)
                    << *multiply ( dx/dt, *computeVorticity (*computeVelocity (lattice), domaine2d));
                // Incrementing vorticity file name
                vorticityFileIndex ++;
            }
        }

        // Trace les profiles de vitesse Vx et Vy
        if (saveProfil>0) {
            if (iT%parameters.nStep(saveProfil)==0) {
                // Measuring points:
    const plint yM = (ny-1)/2;  //Midpoint in y-axis
    const plint x1 = nx-1;
    const plint x0 = 0;
		Box2D profileSectionXaxis1(x1*0, x1*0, yM, yM);

            Box2D profileSectionXaxis2(x1*0.02 , x1*0.02, yM, yM);



            Box2D profileSectionXaxis3(x1*0.04 , x1*0.04, yM, yM);


                Box2D profileSectionXaxis4(x1*0.06 , x1*0.06, yM, yM);


                Box2D profileSectionXaxis5(x1*0.08 , x1*0.08, yM, yM);


                Box2D profileSectionXaxis6(x1*0.10 , x1*0.10, yM, yM);


                Box2D profileSectionXaxis7(x1*0.12 , x1*0.12, yM, yM);


                Box2D profileSectionXaxis8(x1*0.14 , x1*0.14, yM, yM);


                Box2D profileSectionXaxis9(x1*0.16 , x1*0.16, yM, yM);


                Box2D profileSectionXaxis10(x1*0.18 , x1*0.18, yM, yM);



Box2D profileSectionXaxis11(x1*0.20 , x1*0.20, yM, yM);


Box2D profileSectionXaxis12(x1*0.22 , x1*0.22, yM, yM);

Box2D profileSectionXaxis13(x1*0.24 , x1*0.24, yM, yM);


Box2D profileSectionXaxis14(x1*0.26 , x1*0.26, yM, yM);


Box2D profileSectionXaxis15(x1*0.28 , x1*0.28, yM, yM);


Box2D profileSectionXaxis16(x1*0.30 , x1*0.30, yM, yM);


Box2D profileSectionXaxis17(x1*0.32 , x1*0.32, yM, yM);


Box2D profileSectionXaxis18(x1*0.34 , x1*0.34, yM, yM);


Box2D profileSectionXaxis19(x1*0.36 , x1*0.36, yM, yM);


Box2D profileSectionXaxis20(x1*0.38 , x1*0.38, yM, yM);


Box2D profileSectionXaxis21(x1*0.40 , x1*0.40, yM, yM);


Box2D profileSectionXaxis22(x1*0.42 , x1*0.42, yM, yM);


Box2D profileSectionXaxis23(x1*0.44 , x1*0.44, yM, yM);


Box2D profileSectionXaxis24(x1*0.46 , x1*0.46, yM, yM);


Box2D profileSectionXaxis25(x1*0.48 , x1*0.48, yM, yM);



Box2D profileSectionXaxis26(x1*0.50 , x1*0.50, yM, yM);


Box2D profileSectionXaxis27(x1*0.52 , x1*0.52, yM, yM);


Box2D profileSectionXaxis28(x1*0.54 , x1*0.54, yM, yM);


Box2D profileSectionXaxis29(x1*0.56 , x1*0.56, yM, yM);


Box2D profileSectionXaxis30(x1*0.58 , x1*0.58, yM, yM);


Box2D profileSectionXaxis31(x1*0.60 , x1*0.60, yM, yM);


Box2D profileSectionXaxis32(x1*0.62 , x1*0.62, yM, yM);


Box2D profileSectionXaxis33(x1*0.64 , x1*0.64, yM, yM);


Box2D profileSectionXaxis34(x1*0.66 , x1*0.66, yM, yM);


Box2D profileSectionXaxis35(x1*0.68 , x1*0.68, yM, yM);


Box2D profileSectionXaxis36(x1*0.70 , x1*0.70, yM, yM);


Box2D profileSectionXaxis37(x1*0.72 , x1*0.72, yM, yM);


Box2D profileSectionXaxis38(x1*0.74 , x1*0.74, yM, yM);


Box2D profileSectionXaxis39(x1*0.76 , x1*0.76, yM, yM);

Box2D profileSectionXaxis40(x1*0.78 , x1*0.78, yM, yM);


Box2D profileSectionXaxis41(x1*0.80 , x1*0.80, yM, yM);


Box2D profileSectionXaxis42(x1*0.82 , x1*0.82, yM, yM);


Box2D profileSectionXaxis43(x1*0.84 , x1*0.84, yM, yM);


Box2D profileSectionXaxis44(x1*0.86 , x1*0.86, yM, yM);


Box2D profileSectionXaxis45(x1*0.88 , x1*0.88, yM, yM);


Box2D profileSectionXaxis46(x1*0.90 , x1*0.90, yM, yM);


Box2D profileSectionXaxis47(x1*0.92 , x1*0.92, yM, yM);


Box2D profileSectionXaxis48(x1*0.94 , x1*0.94, yM, yM);


Box2D profileSectionXaxis49(x1*0.96 , x1*0.96, yM, yM);


Box2D profileSectionXaxis50(x1*0.98 , x1*0.98, yM, yM);

Box2D profileSectionXaxis51(x1*1 , x1*1, yM, yM);


                xAxisVxProfiles1
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis1, xComponent) )
                    << endl;
                 xAxisVxProfiles2
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis2, xComponent) )
                    << endl;
                     xAxisVxProfiles3
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis3, xComponent) )
                    << endl;
                     xAxisVxProfiles4
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis4, xComponent) )
                    << endl;
                     xAxisVxProfiles5
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis5, xComponent) )
                    << endl;
                     xAxisVxProfiles6
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis6, xComponent) )
                    << endl;
                     xAxisVxProfiles7
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis7, xComponent) )
                    << endl;
                     xAxisVxProfiles8
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis8, xComponent) )
                    << endl;
                     xAxisVxProfiles9
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis9, xComponent) )
                    << endl;
                     xAxisVxProfiles10
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis10, xComponent) )
                    << endl;
                     xAxisVxProfiles11
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis11, xComponent) )
                    << endl;
                     xAxisVxProfiles12
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis12, xComponent) )
                    << endl;
                     xAxisVxProfiles13
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis13, xComponent) )
                    << endl;
                     xAxisVxProfiles14
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis14, xComponent) )
                    << endl;
                     xAxisVxProfiles15
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis15, xComponent) )
                    << endl;
                     xAxisVxProfiles16
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis16, xComponent) )
                    << endl;
                     xAxisVxProfiles17
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis17, xComponent) )
                    << endl;
                     xAxisVxProfiles18
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis18, xComponent) )
                    << endl;
                     xAxisVxProfiles19
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis19, xComponent) )
                    << endl;
                     xAxisVxProfiles20
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis20, xComponent) )
                    << endl;
                     xAxisVxProfiles21
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis21, xComponent) )
                    << endl;
                     xAxisVxProfiles22
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis22, xComponent) )
                    << endl;
                     xAxisVxProfiles23
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis23, xComponent) )
                    << endl;
                     xAxisVxProfiles24
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis24, xComponent) )
                    << endl;
                     xAxisVxProfiles25
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis25, xComponent) )
                    << endl;
                     xAxisVxProfiles26
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis26, xComponent) )
                    << endl;
                     xAxisVxProfiles27
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis27, xComponent) )
                    << endl;
                     xAxisVxProfiles28
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis28, xComponent) )
                    << endl;
                     xAxisVxProfiles29
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis29, xComponent) )
                    << endl;
                     xAxisVxProfiles30
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis30, xComponent) )
                    << endl;
                     xAxisVxProfiles31
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis31, xComponent) )
                    << endl;
                     xAxisVxProfiles32
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis32, xComponent) )
                    << endl;
                     xAxisVxProfiles33
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis33, xComponent) )
                    << endl;
                     xAxisVxProfiles34
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis34, xComponent) )
                    << endl;
                     xAxisVxProfiles35
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis35, xComponent) )
                    << endl;
                     xAxisVxProfiles36
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis36, xComponent) )
                    << endl;
                     xAxisVxProfiles37
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis37, xComponent) )
                    << endl;
                     xAxisVxProfiles38
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis38, xComponent) )
                    << endl;
                     xAxisVxProfiles39
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis39, xComponent) )
                    << endl;
                     xAxisVxProfiles40
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis40, xComponent) )
                    << endl;
                     xAxisVxProfiles41
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis41, xComponent) )
                    << endl;
                     xAxisVxProfiles42
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis42, xComponent) )
                    << endl;
                     xAxisVxProfiles43
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis43, xComponent) )
                    << endl;
                     xAxisVxProfiles44
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis44, xComponent) )
                    << endl;
                     xAxisVxProfiles45
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis45, xComponent) )
                    << endl;
                    xAxisVxProfiles46
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis46, xComponent) )
                    << endl;
                     xAxisVxProfiles47
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis47, xComponent) )
                    << endl;
                     xAxisVxProfiles48
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis48, xComponent) )
                    << endl;
                    xAxisVxProfiles49
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis49, xComponent) )
                    << endl;
                     xAxisVxProfiles50
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis50, xComponent) )
                    << endl;
                     xAxisVxProfiles51
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis51, xComponent) )
                    << endl;


                // Profil 1
                // Definition de la section du trace : l'amont de la cavite
                Box2D profileSection1(xCav-rCorru, xCav-rCorru, y2+1, y3-1);
                //trace Vx
                profilVx1
                    << setprecision(10)
                    << *multiply ( dx/dt, *computeVelocityComponent (lattice, profileSection1, xComponent) )
                    << endl;
                //trace Vy
                profilVy1
                    << setprecision(10)
                    << *multiply ( dx/dt, *computeVelocityComponent (lattice, profileSection1, yComponent) )
                    << endl;
                // Profil 2
                // Definition de la section du trace : la sortie de la cavite
                Box2D profileSection2(xCav+lCav+rCorru, xCav+lCav+rCorru, y2+1, y3-1);
                //trace Vx
                profilVx2
                    << setprecision(10)
                    // (2) Convert from lattice to physical units.
                    << *multiply ( dx/dt, *computeVelocityComponent (lattice, profileSection2, xComponent) )
                    << endl;
                //trace Vy
                profilVy2
                    << setprecision(10)
                    << *multiply ( dx/dt, *computeVelocityComponent (lattice, profileSection2, yComponent) )
                    << endl;
            }
        }

        if ( (iT>0) && (saveCheckpoint>0) ) {
            if (iT%parameters.nStep(saveCheckpoint)==0) {
                pcout << "Saving state of the simulation for iT = " << (iT)
                    << " t=" << iT*dt << endl;
                saveBinaryBlock(lattice, "checkpoint.dat");
                // On efface le precedent "parameters.dat"
                readParameters ( entier, entier, reel, reel, reel, reel, reel, reel,
                    reel, reel, reel, reel, reel, reel, entier, entier, entier, entier );
                // On ecrit le nouveau "parameters.dat"
                writeParameters ( iRun, iT, TMax, logT, saveGif, saveVTK,
                    saveProfil, initVortiStream, saveVortiStream, saveCheckpoint,
                    initImageStream, saveImageStream, initPointStream, savePointStream,
                    gifFileIndex, vtkFileIndex, velocityFileIndex, vorticityFileIndex );
            }
        }

        if (iT%parameters.nStep(logT)==0) {
            // On efface le precedent "parameters.dat"
            readParameters ( entier, entier, TMax, logT, saveGif, saveVTK,
                saveProfil, initVortiStream, saveVortiStream, saveCheckpoint,
                initImageStream, saveImageStream, initPointStream, savePointStream,
                gifFileIndex, vtkFileIndex, velocityFileIndex, vorticityFileIndex );
            // On ecrit le nouveau "parameters.dat"
            writeParameters ( iRun, iT, TMax, logT, saveGif, saveVTK,
                saveProfil, initVortiStream, saveVortiStream, saveCheckpoint,
                initImageStream, saveImageStream, initPointStream, savePointStream,
                gifFileIndex, vtkFileIndex, velocityFileIndex, vorticityFileIndex );
            pcout << "step " << iT
                  << "; t=" << iT * dt
                  << "; av energy ="
                  << setprecision(10) << getStoredAverageEnergy<T>(lattice)
                  << "; av rho ="
                  << getStoredAverageDensity<T>(lattice)
                  <<endl;
        }

        // Lattice Boltzmann iteration step.
        lattice.collideAndStream();

        // At this point, the state of the lattice corresponds to the
       //  discrete time iT+1, and the stored averages are upgraded to time iT.
    } // On incremente iT

    pcout << "Saving the last state of the simulation for iT = " << iT
          << " t=" << iT*dt << endl;

    saveBinaryBlock(lattice, "checkpoint.dat");

    // On efface le precedent "parameters.dat"
    readParameters ( entier, entier, reel, reel, reel, reel, reel, reel,
        reel, reel, reel, reel, reel, reel, entier, entier, entier, entier );

    // On ecrit le nouveau "parameters.dat"
    iRun = iRun + 1;
    writeParameters ( iRun, iT, TMax, logT, saveGif, saveVTK, saveProfil, initVortiStream,
        saveVortiStream, saveCheckpoint, initImageStream, saveImageStream, initPointStream,
        savePointStream, gifFileIndex, vtkFileIndex, velocityFileIndex, vorticityFileIndex );

    pcout << "Program ends normally at iT = " << iT << endl;

    delete boundaryCondition;
}
