/* Anders BK: Notes and questions file */

/* This file is part of the Palabos library.
 * Copyright (C) 2009, 2010 Jonas Latt
 * E-mail contact: jonas@lbmethod.org
 * The most recent release of Palabos can be downloaded at
 * <http://www.lbmethod.org/palabos/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/** file
 * Flow through a 2D pipe having a cavity close to the inlet.
 *This example makes use of bounce-back nodes to
 * describe the shape of the rounded pipe and cavity edges.
 * The outlet is modeled through a
 * Neumann (zero velocity-gradient) condition.
 * Modeled by Daniel Mazzoni may/june 2011.
 */

#include "palabos2D.h"
#include "palabos2D.hh"
#include <vector>
#include <cmath>
#include <iostream>
#include <fstream>


#include <string>


using namespace plb;
using namespace plb::descriptors;
using namespace std;

typedef double T;
const T pi = (T)4.0*atan(1.0) ;
plint iT;

#define DESCRIPTOR D2Q9Descriptor

// Pulse of the flow velocity
T pulsation(T iT, T amplitude, T frequence, IncomprFlowParam<T> const& parameters) {
    T uMax = parameters.getLatticeU() ;
    T t    = iT*parameters.getDeltaT();
	return amplitude * cos(2 * pi* frequence * t) ;
}

//Turbulent flow in a circular tube with smooth walls
// http://ielnx1.epfl.ch/e-lin/Ryhming/documents/chapters/documents_published/doc6/node226.html
// Vx = ( y/a )^(1/nn) nn =  6 si Re =     4 000 ( a = rayon du tube )
//                     nn =  7 si Re =   100 000
//                     nn = 10 si Re = 2 000 000
T turbulentFlowVelicity(plint composante,plint iX, plint iY, IncomprFlowParam<T> const& parameters,
		                plint xTub, plint yTub,plint yTubb,plint ymTubb, plint xCav, plint lCav, T amplitude, T frequence) {
    plint nx = parameters.getNx();
    plint ny = parameters.getNy();
    T nn = 6.;        // Profile for exhibit turbulent
    T mm = 6.;        // Exponent for the connection between the input section and the tube
    T x0 = 0.;        // Abscissa upstream connection speed for the exponential
	T x1 = (T)xTub;   // Abscissa downstream connection speed for exponential
    T y0 = 0.;        // Ordinate upstream connection speed for exponential
    T y1 = (T)yTub;   // Y-connection downstream of the exponential for speed
	T y2 = (T)(ny-1-yTub); // Upper limit. of flow in the tube
	T y3 = (T)(ny-1);
	T y5=(T)(ny-1-yTubb);      // Upper limit. of flow in the tube
     T y4 = (T)yTubb;

    T xx = (T)iX;
    T yy = (T)iY;
    T yx = (xx<x1) ? y0+(y1-y0)/2.*(1.+sin(pi*(pow((xx-x0),mm)/pow((x1-x0),mm)-0.5))) : y1;
	                //Particle trajectory at the entrance of the tube
    T yp = (xx<x1) ? (y1-y0)/2. * mm * pi * pow(xx-x0,mm-1.) / pow(x1-x0,mm)
		 * cos(pi*(pow(xx-x0,mm)/pow(x1-x0,mm)-0.5)) : 0.; // Derivative of the trajectory
	T rs = (y2-y1)/(y3-y0-2.*yx) ; // rs<=1 = Rpport (Sections tube) / (Section a l'ascisse x) = S(x1) / S(x)
	T vx = rs * parameters.getLatticeU() ; // U(x) = uMax * ( S(x1) / S(x) )
	T vy = yp * parameters.getLatticeU() * (y3+y0-2.*yy)/(y3-y0); // vy change de signe entre y0 et y3, vy changes sign between
	T yc = (yy<((yx+(y3-yx))/2.)) ? yy-yx : (y3-yx)-yy;
	T up = ((yy>yx)&&(yy<(y3-yx))) ? pow(2.*yc/((y3-yx)-yx),(1./nn)) : 0; // Boundary layer profile
	//T up = ((yy>yx*0.)&&(yy<(y3-yx*0.))) ? (2.*yc*yx*0.+yc*yc)/(y3*y3/4. -yx*yx*0.) : 0; // top hat profile
	T vitesse = (composante==0) ? vx : vy ;

	// pcout<<"  6  "<<endl; // ZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::string sinputString;
	// std::cin >> sinputString;

    T t = iT * parameters.getDeltaT();

    return up * vitesse;


    //return 0.5*sin(8*atan(1)*34.56*t);                     // test 3

    //return up * (vitesse + 0.5*sin(8*atan(1)*34.56*t)) ;  // 1

    //return up * vitesse * (1. + 0.5*sin(8*atan(1)*34.56*t)) ;  //Kjørt i win (mode3)
    //return up * vitesse * (1. + 10*sin(8*atan(1)*34.56*t));  //4

    //return up * vitesse * 4 *.(1. + 0.5*sin(8*atan(1)*34.56*t)); // neste?

    //return 0.5*sin(8*atan(1)*34.56*t) * parameters.getLatticeU();  //  Denne kjøres i win 8feb morgen.
}

// Linearly decreasing pressure profile
T turbulentFlowPressure(plint iX, IncomprFlowParam<T> const& parameters) {
    T Lx = parameters.getNx()-1;
    T Ly = parameters.getNy()-1;
    return 8.*parameters.getLatticeNu()*parameters.getLatticeU() / (Ly*Ly) * (Lx/(T)2-(T)iX);
}

// Convert pressure to density according to ideal gas law
T turbulentFlowDensity(plint iX, IncomprFlowParam<T> const& parameters) {
    return turbulentFlowPressure(iX,parameters)*DESCRIPTOR<T>::invCs2 + (T)1;
}

// A functional, used to initialize the velocity for the boundary conditions
template<typename T>
class TurbulentFlowVelocity {
public:
    TurbulentFlowVelocity(IncomprFlowParam<T> parameters_,
			plint xTub_, plint yTub_,plint yTubb_, plint ymTubb_, plint xCav_, plint lCav_, T amplitude_, T frequence_)
        : parameters(parameters_), xTub(xTub_), yTub(yTub_),yTubb(yTubb_), ymTubb(ymTubb_), xCav(xCav_), lCav(lCav_),
		amplitude(amplitude_), frequence(frequence_)
    { }
    void operator()(plint iX, plint iY, Array<T,2>& u) const {
        u[0] = turbulentFlowVelicity(0, iX, iY, parameters, xTub, yTub,yTubb, ymTubb, xCav, lCav, amplitude, frequence);
        u[1] = turbulentFlowVelicity(1, iX, iY, parameters, xTub, yTub,yTubb, ymTubb, xCav, lCav, amplitude, frequence);
    }
private:
    IncomprFlowParam<T> parameters;
	T amplitude, frequence;
	plint xTub, yTub,yTubb,ymTubb, xCav, lCav;
};

// A functional, used to initialize a pressure boundary to constant density
template<typename T>
class ConstantDensity {
public:
    ConstantDensity(T density_)
        : density(density_)
    { }
    T operator()(plint iX, plint iY) const {
        return density;
    }
private:
    T density;
};

// A functional, used to create an initial condition for the density and velocity
template<typename T>
class TurbulentFlowVelocityAndDensity {
public:
    TurbulentFlowVelocityAndDensity(IncomprFlowParam<T> parameters_,
			plint xTub_, plint yTub_,plint yTubb_, plint ymTubb_, plint xCav_, plint lCav_, T amplitude_, T frequence_)
        : parameters(parameters_), xTub(xTub_), yTub(yTub_),yTubb(yTubb_),ymTubb(ymTubb_), xCav(xCav_), lCav(lCav_),
		amplitude(amplitude_), frequence(frequence_)
	{ }
    void operator()(plint iX, plint iY, T& rho, Array<T,2>& u) const {
        rho  = turbulentFlowDensity(iX,parameters);
        u[0] = turbulentFlowVelicity(0, iX, iY, parameters, xTub, yTub,yTubb, ymTubb, xCav, lCav, amplitude, frequence);
        u[1] = turbulentFlowVelicity(1, iX, iY, parameters, xTub, yTub,yTubb, ymTubb, xCav, lCav, amplitude, frequence);
    }
private:
    IncomprFlowParam<T> parameters;
	T amplitude, frequence;
	plint xTub, yTub,yTubb, ymTubb, xCav, lCav;
};

// A functional, used to instantiate bounce-back nodes at the locations of the cylinder
template<typename T>
class CylinderShapeDomain2D : public plb::DomainFunctional2D {
public:
    CylinderShapeDomain2D(plb::plint cx_, plb::plint cy_, plb::plint radius)
        : cx(cx_),
          cy(cy_),
          radiusSqr(plb::util::sqr(radius))
    { }
    virtual bool operator() (plb::plint iX, plb::plint iY) const {
        return plb::util::sqr(iX-cx) + plb::util::sqr(iY-cy) <= radiusSqr;
    }
    virtual CylinderShapeDomain2D<T>* clone() const {
        return new CylinderShapeDomain2D<T>(*this);
    }
private:
    plb::plint cx;
    plb::plint cy;
    plb::plint radiusSqr;
};

// A functional, used to instantiate bounce-back nodes at the locations of a rectangle
template<typename T>
class RectangleShapeDomain2D : public plb::DomainFunctional2D {
public:
    RectangleShapeDomain2D(plb::plint x0_, plb::plint x1_, plb::plint y0_, plb::plint y1_)
        : x0(x0_),
          x1(x1_),
          y0(y0_),
          y1(y1_)
    { }
    virtual bool operator() (plb::plint iX, plb::plint iY) const {
		T dif1 = x0-iX;
		T dif2 = x1-iX;
		T dif3 = y0-iY;
		T dif4 = y1-iY;
		return (dif1 < 0)&&(dif2 > 0)&&(dif3 < 0)&&(dif4 > 0);
    }
    virtual RectangleShapeDomain2D<T>* clone() const {
        return new RectangleShapeDomain2D<T>(*this);
    }
private:
    plb::plint x0;
    plb::plint x1;
    plb::plint y0;
    plb::plint y1;
};

void boundaryConditionsSetup( MultiBlockLattice2D<T,DESCRIPTOR>& lattice,
                              IncomprFlowParam<T> const& parameters,
                              OnLatticeBoundaryCondition2D<T,DESCRIPTOR>& boundaryCondition,
							  plint xTub, plint yTub, plint yTubb, plint ymTubb, plint xCav, plint lCav, T amplitude, T frequence)
{
    const plint nx = parameters.getNx();
    const plint ny = parameters.getNy();

    plint x0 = 0;
    plint x1 = nx-1;

    plint xtub = xTub; // Upstream edge of the cavity
    plint xcav = xCav; // Upstream edge of the cavity
    plint lcav = lCav; // Length of the cavity

    plint rayon1 = 10;  // Rounded edges of cavity
    plint demiR2 = yTubb/4; //plint demiR2 = yTub/4;
    plint rayon2 = 2*demiR2;

    plint y0 = 0;  //0
    plint y1 = yTub;
    plint y4 = yTubb;
    plint y5 = ymTubb;
	plint y2 = ny-1-yTub;
	plint y3 = ny-1;

    //pcout<<"  4  "<<endl; // ZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::string sinputString;
	// std::cin >> sinputString;

    // The Neumann condition at the output speed
    Box2D outlet(nx-1, nx-1, ny/2+1, ny-2);


    // Create Velocity boundary conditions everywhere
    boundaryCondition.setVelocityConditionOnBlockBoundaries (
            lattice, Box2D(0, 0, 1, ny-2) );
    boundaryCondition.setVelocityConditionOnBlockBoundaries (
            lattice, Box2D(0, nx-1, 0, 0) );
    boundaryCondition.setVelocityConditionOnBlockBoundaries (
            lattice, Box2D(0, nx-1, ny-1, ny-1) );

    // .. except on right boundary, where we prefer an outflow condition
    //    (zero velocity-gradient).
    boundaryCondition.setVelocityConditionOnBlockBoundaries (
            lattice, Box2D(nx-1, nx-1, 1, ny-2), boundary::outflow );

	// pcout<<"  5  "<<endl; // ZZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::cin >> sinputString;

    // The condition of speed limits in speed is given by TurbulentFlowVelocity
    setBoundaryVelocity (
            lattice, lattice.getBoundingBox(),
            TurbulentFlowVelocity<T>(parameters, xTub, yTub, yTubb,ymTubb, xCav, lCav, amplitude, frequence) );

	// pcout<<"  7  "<<endl; // ZZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::cin >> sinputString;

    setBoundaryDensity (
            lattice, outlet,
            ConstantDensity<T>(1.) );

	// pcout<<"  8  "<<endl; // ZZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::cin >> sinputString;

    initializeAtEquilibrium (
            lattice, lattice.getBoundingBox(),
            TurbulentFlowVelocityAndDensity<T>(parameters, xTub, yTub, yTubb,ymTubb, xCav, lCav, amplitude, frequence) );

	// pcout<<"  9  "<<endl; // ZZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::cin >> sinputString;

	//consturction of the geometry

    //Definingthe tube intake with rounded edge (1) and tube (2):
    // (1) We define the rounded upstream of the tub:
    // Using radius rayon 2:
defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(xtub+rayon1,y1-rayon1,rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new
				   RectangleShapeDomain2D<T>(xtub,xtub+rayon1,y0,y1-rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

     // Using radius rayon 1:
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(xtub+rayon1,y2+rayon1,rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xtub,xtub+rayon1,y2+rayon1,y3),
                   new plb::BounceBack<T,DESCRIPTOR>);

    // (2) We define the portion of tube upstream of the cavity
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xtub+rayon1,xcav-rayon1,y0,y1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xtub+rayon1,xcav-rayon1,y2,y3),
                   new plb::BounceBack<T,DESCRIPTOR>);

/////////////////////////////

    // Defining the cavity:
    // We define the rounded upstream of the cavity
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(xcav-rayon1,y1-rayon1,rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   // new RectangleShapeDomain2D<T>(xcav-2*rayon1,xcav,y0,y1-rayon1),
                   new RectangleShapeDomain2D<T>(xcav-rayon1,xcav,y0,y1-rayon1),  // why xcav-2*rayon1 and not xcav-rayon1?
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(xcav-rayon1,y2+rayon1,rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   //new RectangleShapeDomain2D<T>(xcav-2*rayon1,xcav,y2+rayon1,y3),
                   new RectangleShapeDomain2D<T>(xcav-rayon1,xcav,y2+rayon1,y3),
                   new plb::BounceBack<T,DESCRIPTOR>);
                        // why xcav-2*rayon1 and not xcav-rayon1?

    //Defining the cavity space:
    //Already done by BC


    // We define the rounding down of the cavity
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(xcav+lcav+rayon1,y1-rayon1,rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xcav+lcav,xcav+lcav+2*rayon1,y0,y1-rayon1),//                   new RectangleShapeDomain2D<T>(xcav+lcav,xcav+lcav+2*rayon1,y0,y1-rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(xcav+lcav+rayon1,y2+rayon1,rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xcav+lcav,xcav+lcav+2*rayon1,y2+rayon1,y3),
                   new plb::BounceBack<T,DESCRIPTOR>);

    // We define the tube portion downstream of the cavity
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xcav+lcav+rayon1,x1,y0,y1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xcav+lcav+rayon1,x1,y2,y3),
                   new plb::BounceBack<T,DESCRIPTOR>);
     //We define the depth of the cavity
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xcav,xcav+lcav,y0,y4),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xcav,xcav+lcav,y5,y3),
                   new plb::BounceBack<T,DESCRIPTOR>);
	// For a single tube :
    // defineDynamics(lattice, lattice.getBoundingBox(),
    //                new RectangleShapeDomain2D<T>(x0,x1,y0,y1),
    //                new plb::BounceBack<T,DESCRIPTOR>);

	// pcout<<"  10  "<<endl; // ZZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::cin >> sinputString;

    lattice.initialize();
}

void writeGif(MultiBlockLattice2D<T,DESCRIPTOR>& lattice,IncomprFlowParam<T> const& parameters, plint iter)
{
    const plint imSize = 800;
    const plint nx = parameters.getNx();
    const plint ny = parameters.getNy();
    const plint rCorru = 10;     // Arrondi des bords de cavite
    const plint rInlet = 10;     // Arrondi de l'entree d'air

    const plint nCav =   1;      //   Le nombre de cavites

    // Dimensions en unite physiques (mm)
    const plint h_Dom = 30;           //  hauteur totale totale du maillage
    const plint x_Pip = 20;           //  abscisse de l'entree du tube
    const plint y_Inl =  5;           //  Ordonnee du premier tube (inlet )
    const plint y_Pip = 10;           //  Ordonnee du second tube ( riser )
    const plint l_Inl = 10;           //  longueur de tube en amont de la cavite
    const plint l_Cav = 10;           //  longueur de la cavite
    const plint l_Cor =  0;           //  longueur de la corrugation (s'il y a plus d'une cavite)
    const plint x_Sil = 70;           //  abscisse du second silencieux

    const plint l_Dom = (plint)(0.5+(T)(nx*h_Dom)/(T) ny);   //  Longueur totale du maillage
    const plint l_Tot = l_Dom - x_Pip; //  Longueur totale du tube
    const plint x_Cav = l_Inl + x_Pip; //  Abscisse de la cavite

    // Dimensions en unite maillage
    const plint xPip = (plint)(0.5+x_Pip*(ny-1)/h_Dom);
    const plint yInl = (plint)(0.5+y_Inl*(ny-1)/h_Dom);
    const plint yPip = (plint)(0.5+y_Pip*(ny-1)/h_Dom);
    const plint xCav = (plint)(0.5+x_Cav*(ny-1)/h_Dom);
    const plint lCav = (plint)(0.5+l_Cav*(ny-1)/h_Dom);
    const plint lCor = (plint)(0.5+l_Cor*(ny-1)/h_Dom);
    const plint xSil = (plint)(0.5+x_Sil*(ny-1)/h_Dom);

    // Definition des abscisses des 2 SparceBlocks [x0, x1] et [x1+1, x2]
    const plint x0 = 0;
    const plint x1 = (xPip+xCav)/2;
    const plint x2 = xCav+2*lCav;
    const plint x3 = nx-1;

    // Definition des ordonnees des 2 SparceBlocks
    const plint y0 = 0;
    const plint y5 = ny-1;
    const plint y1 = yInl;
    const plint y2 = yPip;
    const plint y3 = y5-y2;
    const plint y4 = y5-y1;
// Create a block distribution with the three added blocks.
 /*   plint envelopeWidth = 1;
    SparseBlockStructure2D sparseBlock(nx, ny);
    sparseBlock.addBlock(Box2D(x0, x1-1, y2, y3), sparseBlock.nextIncrementalId());
    sparseBlock.addBlock(Box2D(x1, x2-1, y2, y3), sparseBlock.nextIncrementalId());
    sparseBlock.addBlock(Box2D(x2, x3, y2, y3), sparseBlock.nextIncrementalId());//sparseBlock.addBlock(Box2D(xb2, xb3-1, y0, y5), sparseBlock.nextIncrementalId());
*/
    const plint nbpX = 100; // Pour la sortie de fieldPoint1_Vx
    const plint nbpY =  50; // Pour la sortie de fieldPoint2_Vx et fieldPoint3_Vx



    const T dx = parameters.getDeltaX();
    const T dt = parameters.getDeltaT();
   // Box2D outlet(xCav+1   -((y2+y3)/2-lCav)/2, xCav+lCav+((y2+y3)/2-lCav)/2, y0-1, y4+1) ;   //y0+1,(y2+y3)/2);
    Box2D outlet(xCav+1   -((y2+y3)/2-lCav)/5, 1.4*xCav+lCav+((y2+y3)/2-lCav)/2, y1/1.5, (y2+y3)/2.5) ;
    Box2D outlett(xCav+1   -((y2+y3)/2-lCav)/6, xCav+lCav+((y2+y3)/2-lCav)/2, y1/1.5, (y2+y3)/2.5) ;
    ImageWriter<T> imageWriter("leeloo");
    imageWriter.writeScaledGif(createFileName("u", iter, 6),
                               *computeVelocityNorm(lattice),
                               imSize, imSize );
    ImageWriter<T>ImageWriter("fire");
    ImageWriter.writeScaledGif( createFileName("omega", iter, 6),
                                (*computeVorticity (
                                        *computeVelocity(lattice, outlet)) ),
                                imSize, imSize );

    ImageWriter.writeScaledGif( createFileName("omegga", iter, 6),
                                (*computeVorticity (
                                        *computeVelocity(lattice, outlett)) ),
                                imSize, imSize );
}





void writeVTK(MultiBlockLattice2D<T,DESCRIPTOR>& lattice,
              IncomprFlowParam<T> const& parameters, plint iter)
{
    T dx = parameters.getDeltaX();
    T dt = parameters.getDeltaT();
    VtkImageOutput2D<T> vtkOut(createFileName("vtk", iter, 6), dx);
    vtkOut.writeData<float>(*computeVelocityNorm(lattice), "velocityNorm", dx/dt);
    vtkOut.writeData<2,float>(*computeVelocity(lattice), "velocity", dx/dt);
}

/* Definition de la geometrie

+===========+      +=============+-------------------------y3
            |      |             |
            |      |             |
            o======o             o======================+--y2


----> U


            o======o             o======================+--y1
            |      |             |
            |      |             |
+===========+      +=============+-------------------------y0=0
|           |      |             |                      |
x0=0      xTub   xCav       xCav+lCav               x1=nx-1
*/
int main(int argc, char* argv[]) {
    plbInit(&argc, &argv);

    IncomprFlowParam<T> parameters(
            (T) 0.017, // c=1/sqrt(3) => u_LB = u_SI/sqrt(3)/340  //0.0136
            (T) 5800, // Re = for a tube diameter of 1 cm. (Reynolds Number)
            500,      // N
            4.,    // lx 7.2
            1.      // ly 1.2
    );
    const plint nx = parameters.getNx();
    const plint ny = parameters.getNy();

     pcout << "nx" << nx << endl;
     pcout << "ny" << ny << endl;
    pcout << "Omega is " << parameters.getOmega() << endl;
    pcout << "Tau is " << 1/parameters.getOmega() << endl;

    const plint rayon1 = 10;     // Rounded edges of cavity

    const plint y0 = 0;
	const plint y3 = ny-1;
    const plint y1 = ny/4;
	const plint y2 = y3-y1;

    const plint xTub = ny;
    const plint yTub = ny/4;
    const plint yTubb = yTub*2/3;
    const plint ymTubb =ny-yTubb;
    const plint xCav = xTub+ny/2;
    const plint lCav = ny/4;  // ny/4

    const T maxT        = (T)30.1;  //30.1
    const T iTMin              = (plint)0 ;
    const T logT        = (T)0.01;    // logging interval?
    const T imSave      = (T)0.05;    //0.05 interval of image save?
    const T vtkSave     = (T)20.;      // interval of data save?
    const T profileSave = (T)0.01;
    plint gifFileIndex  = (plint) 0;
    //const T saveVorticity=(T)0.05;
    //const T initVorticity= (T)0.0;

    T amplitude = 1.; //fra 1.
    T frequence = 1.;  // fra 1.

    // Measuring points:
    const plint yM = ny/2;  //Midpoint in y-axis
    const plint x1 = nx-1;
    const plint x0 = 0;

    pcout << "delta X is " << parameters.getDeltaX() << endl;
    pcout << "delta T is " << parameters.getDeltaT() << endl;
    pcout << "deltaX/deltaT is " << parameters.getDeltaX() / parameters.getDeltaT() << endl;
    pcout << "ymTub is " << ymTubb << endl;
    pcout << "yTub is " << yTubb << endl;

	// Output directory
    global::directories().setOutputDir("./tmp/");
    // The following command opens a text-file, in which
    // the velocity-profile is written at successive time steps.
    //plb_ofstream Vx1Profiles("tmp/Vx1Profiles.dat");
    //plb_ofstream Vy1Profiles("tmp/Vy1Profiles.dat");
    //plb_ofstream Vx2Profiles("tmp/Vx2Profiles.dat");
    //plb_ofstream Vy2Profiles("tmp/Vy2Profiles.dat");
    plb_ofstream xAxisVxProfiles1("tmp/xAxisVxProfiles1.dat");
    plb_ofstream xAxisVxProfiles2("tmp/xAxisVxProfiles2.dat");
    plb_ofstream xAxisVxProfiles3("tmp/xAxisVxProfiles3.dat");
    plb_ofstream xAxisVxProfiles4("tmp/xAxisVxProfiles4.dat");
    plb_ofstream xAxisVxProfiles5("tmp/xAxisVxProfiles5.dat");
    plb_ofstream xAxisVxProfiles6("tmp/xAxisVxProfiles6.dat");
    plb_ofstream xAxisVxProfiles7("tmp/xAxisVxProfiles7.dat");
    plb_ofstream xAxisVxProfiles8("tmp/xAxisVxProfiles8.dat");
    plb_ofstream xAxisVxProfiles9("tmp/xAxisVxProfiles9.dat");
    plb_ofstream xAxisVxProfiles10("tmp/xAxisVxProfiles10.dat");
    plb_ofstream xAxisVxProfiles11("tmp/xAxisVxProfiles11.dat");
    plb_ofstream xAxisVxProfiles12("tmp/xAxisVxProfiles12.dat");
    plb_ofstream xAxisVxProfiles13("tmp/xAxisVxProfiles13.dat");
    plb_ofstream xAxisVxProfiles14("tmp/xAxisVxProfiles14.dat");
    plb_ofstream xAxisVxProfiles15("tmp/xAxisVxProfiles15.dat");
    plb_ofstream xAxisVxProfiles16("tmp/xAxisVxProfiles16.dat");
    plb_ofstream xAxisVxProfiles17("tmp/xAxisVxProfiles17.dat");
    plb_ofstream xAxisVxProfiles18("tmp/xAxisVxProfiles18.dat");
    plb_ofstream xAxisVxProfiles19("tmp/xAxisVxProfiles19.dat");
    plb_ofstream xAxisVxProfiles20("tmp/xAxisVxProfiles20.dat");
    plb_ofstream xAxisVxProfiles21("tmp/xAxisVxProfiles21.dat");
    plb_ofstream xAxisVxProfiles22("tmp/xAxisVxProfiles22.dat");
    plb_ofstream xAxisVxProfiles23("tmp/xAxisVxProfiles23.dat");
    plb_ofstream xAxisVxProfiles24("tmp/xAxisVxProfiles24.dat");
    plb_ofstream xAxisVxProfiles25("tmp/xAxisVxProfiles25.dat");
    plb_ofstream xAxisVxProfiles26("tmp/xAxisVxProfiles26.dat");
    plb_ofstream xAxisVxProfiles27("tmp/xAxisVxProfiles27.dat");
    plb_ofstream xAxisVxProfiles28("tmp/xAxisVxProfiles28.dat");
    plb_ofstream xAxisVxProfiles29("tmp/xAxisVxProfiles29.dat");
    plb_ofstream xAxisVxProfiles30("tmp/xAxisVxProfiles30.dat");
    plb_ofstream xAxisVxProfiles31("tmp/xAxisVxProfiles31.dat");
    plb_ofstream xAxisVxProfiles32("tmp/xAxisVxProfiles32.dat");
    plb_ofstream xAxisVxProfiles33("tmp/xAxisVxProfiles33.dat");
    plb_ofstream xAxisVxProfiles34("tmp/xAxisVxProfiles34.dat");
    plb_ofstream xAxisVxProfiles35("tmp/xAxisVxProfiles35.dat");
    plb_ofstream xAxisVxProfiles36("tmp/xAxisVxProfiles36.dat");
    plb_ofstream xAxisVxProfiles37("tmp/xAxisVxProfiles37.dat");
    plb_ofstream xAxisVxProfiles38("tmp/xAxisVxProfiles38.dat");
    plb_ofstream xAxisVxProfiles39("tmp/xAxisVxProfiles39.dat");
    plb_ofstream xAxisVxProfiles40("tmp/xAxisVxProfiles40.dat");
    plb_ofstream xAxisVxProfiles41("tmp/xAxisVxProfiles41.dat");
    plb_ofstream xAxisVxProfiles42("tmp/xAxisVxProfiles42.dat");
    plb_ofstream xAxisVxProfiles43("tmp/xAxisVxProfiles43.dat");
    plb_ofstream xAxisVxProfiles44("tmp/xAxisVxProfiles44.dat");
    plb_ofstream xAxisVxProfiles45("tmp/xAxisVxProfiles45.dat");
    plb_ofstream xAxisVxProfiles46("tmp/xAxisVxProfiles46.dat");
    plb_ofstream xAxisVxProfiles47("tmp/xAxisVxProfiles47.dat");
    plb_ofstream xAxisVxProfiles48("tmp/xAxisVxProfiles48.dat");
    plb_ofstream xAxisVxProfiles49("tmp/xAxisVxProfiles49.dat");
    plb_ofstream xAxisVxProfiles50("tmp/xAxisVxProfiles50.dat");
    plb_ofstream xAxisVxProfiles51("tmp/xAxisVxProfiles51.dat");


    //plb_ofstream pressureProfiles("tmp/xAxisPressureProfiles.dat");
    plb_ofstream densityProfiles1("tmp/xAxisDensityProfiles1.dat");
    plb_ofstream densityProfiles2("tmp/xAxisDensityProfiles2.dat");
    plb_ofstream densityProfiles3("tmp/xAxisDensityProfiles3.dat");
    plb_ofstream densityProfiles4("tmp/xAxisDensityProfiles4.dat");
    plb_ofstream densityProfiles5("tmp/xAxisDensityProfiles5.dat");
    plb_ofstream densityProfiles6("tmp/xAxisDensityProfiles6.dat");
    plb_ofstream densityProfiles7("tmp/xAxisDensityProfiles7.dat");
    plb_ofstream densityProfiles8("tmp/xAxisDensityProfiles8.dat");
    plb_ofstream densityProfiles9("tmp/xAxisDensityProfiles9.dat");
    plb_ofstream densityProfiles10("tmp/xAxisDensityProfiles10.dat");
    plb_ofstream densityProfiles11("tmp/xAxisDensityProfiles11.dat");
    plb_ofstream densityProfiles12("tmp/xAxisDensityProfiles12.dat");
    plb_ofstream densityProfiles13("tmp/xAxisDensityProfiles13.dat");
    plb_ofstream densityProfiles14("tmp/xAxisDensityProfiles14.dat");
    plb_ofstream densityProfiles15("tmp/xAxisDensityProfiles15.dat");
    plb_ofstream densityProfiles16("tmp/xAxisDensityProfiles16.dat");
    plb_ofstream densityProfiles17("tmp/xAxisDensityProfiles17.dat");
    plb_ofstream densityProfiles18("tmp/xAxisDensityProfiles18.dat");
    plb_ofstream densityProfiles19("tmp/xAxisDensityProfiles19.dat");
    plb_ofstream densityProfiles20("tmp/xAxisDensityProfiles20.dat");
    plb_ofstream densityProfiles21("tmp/xAxisDensityProfiles21.dat");
    plb_ofstream densityProfiles22("tmp/xAxisDensityProfiles22.dat");
    plb_ofstream densityProfiles23("tmp/xAxisDensityProfiles23.dat");
    plb_ofstream densityProfiles24("tmp/xAxisDensityProfiles24.dat");
    plb_ofstream densityProfiles25("tmp/xAxisDensityProfiles25.dat");
    plb_ofstream densityProfiles26("tmp/xAxisDensityProfiles26.dat");
    plb_ofstream densityProfiles27("tmp/xAxisDensityProfiles27.dat");
    plb_ofstream densityProfiles28("tmp/xAxisDensityProfiles28.dat");
    plb_ofstream densityProfiles29("tmp/xAxisDensityProfiles29.dat");
    plb_ofstream densityProfiles30("tmp/xAxisDensityProfiles30.dat");
    plb_ofstream densityProfiles31("tmp/xAxisDensityProfiles31.dat");
    plb_ofstream densityProfiles32("tmp/xAxisDensityProfiles32.dat");
    plb_ofstream densityProfiles33("tmp/xAxisDensityProfiles33.dat");
    plb_ofstream densityProfiles34("tmp/xAxisDensityProfiles34.dat");
    plb_ofstream densityProfiles35("tmp/xAxisDensityProfiles35.dat");
    plb_ofstream densityProfiles36("tmp/xAxisDensityProfiles36.dat");
    plb_ofstream densityProfiles37("tmp/xAxisDensityProfiles37.dat");
    plb_ofstream densityProfiles38("tmp/xAxisDensityProfiles38.dat");
    plb_ofstream densityProfiles39("tmp/xAxisDensityProfiles39.dat");
    plb_ofstream densityProfiles40("tmp/xAxisDensityProfiles40.dat");
    plb_ofstream densityProfiles41("tmp/xAxisDensityProfiles41.dat");
    plb_ofstream densityProfiles42("tmp/xAxisDensityProfiles42.dat");
    plb_ofstream densityProfiles43("tmp/xAxisDensityProfiles43.dat");
    plb_ofstream densityProfiles44("tmp/xAxisDensityProfiles44.dat");
    plb_ofstream densityProfiles45("tmp/xAxisDensityProfiles45.dat");
    plb_ofstream densityProfiles46("tmp/xAxisDensityProfiles46.dat");
    plb_ofstream densityProfiles47("tmp/xAxisDensityProfiles47.dat");
    plb_ofstream densityProfiles48("tmp/xAxisDensityProfiles48.dat");
    plb_ofstream densityProfiles49("tmp/xAxisDensityProfiles49.dat");
    plb_ofstream densityProfiles50("tmp/xAxisDensityProfiles50.dat");
    plb_ofstream densityProfiles51("tmp/xAxisDensityProfiles51.dat");

     // Instantiate the multi-block, based on the created block distribution on default parameters.
    // with BGK dynamics
    /*writeLogFile(parameters, "BGK dynamics");
    MultiBlockLattice2D<T, DESCRIPTOR> lattice (
        MultiBlockManagement2D ( sparseBlock, defaultMultiBlockPolicy2D().getThreadAttribution(),
            envelopeWidth ),
        defaultMultiBlockPolicy2D().getBlockCommunicator(),
        defaultMultiBlockPolicy2D().getCombinedStatistics(),
        defaultMultiBlockPolicy2D().getMultiCellAccess<T,DESCRIPTOR>(),
        new BGKdynamics<T,DESCRIPTOR>(parameters.getOmega())
    );
     */// End instantiate BGK dynamics.

    // Instantiate the multi-block, based on the created block distribution on default parameters.
    // with Regularized BGK dynamics

     // End instantiate Regularized BGK dynamics.
   // plb_ofstream Vorticity("tmp/Vorticity.dat",ios::app);
   /* const plint boxVortx1 = xCav+1   -((y2+y3)/2-lCav)/5; xCav+1   -((y2+y3)/2-lCav)/6
    const plint boxVortx2 = xCav+lCav+((y2+y3)/2-lCav)/2;
    const plint boxVorty1 = y1/1.5;
    const plint boxVorty2 = (y2+y3)/2;
    */
    const plint boxVortx1 = xCav+1   -((y2+y3)/2-lCav)/6;
    const plint boxVortx2 =  xCav+lCav+((y2+y3)/2-lCav)/2;
    const plint boxVorty1 =  y1/1.5;
    const plint boxVorty2 =  (y2+y3)/2.5;

    const plint boxVortx12 = xCav+1   -((y2+y3)/2-lCav)/5;
    const plint boxVortx22 =  1.4*xCav+lCav+((y2+y3)/2-lCav)/2;
    const plint boxVorty12 =  y1/1.5;
    const plint boxVorty22 =  (y2+y3)/2.5;

    pcout << "boxVortx1 is " << boxVortx1 << endl;
    pcout << "boxVortx2 is " << boxVortx2 << endl;
    pcout << "boxVorty1 is " << boxVorty1 << endl;
    pcout << "boxVorty2 is " << boxVorty2 << endl;

    pcout << "zoomboxVortx1 is " << boxVortx12 << endl;
    pcout << "zoomboxVortx2 is " << boxVortx22 << endl;
    pcout << "zoomboxVorty1 is " << boxVorty12 << endl;
    pcout << "zoomboxVorty2 is " << boxVorty22 << endl;
     //profile
    // Compute only the x-direction of the velocity (direction of the flow).
    plint xComponent = 0;

    // Compute only the y-direction of the velocity (normal to the direction of the flow).
    plint yComponent = 1;

    writeLogFile(parameters, "Turbulent flow");

	// pcout << "  1  "<<endl; // ZZZZZZ
	// std::string sinputString;
	// std::cin >> sinputString;

    MultiBlockLattice2D<T, DESCRIPTOR> lattice (
            parameters.getNx(), parameters.getNy(),
            new RegularizedBGKdynamics<T,DESCRIPTOR>(parameters.getOmega()) );

	// pcout << "  2  "<<endl; // ZZZZZZ
	// std::cin >> sinputString;

    OnLatticeBoundaryCondition2D<T,DESCRIPTOR>*
        boundaryCondition = createLocalBoundaryCondition2D<T,DESCRIPTOR>();

	// pcout<<"  3  "<<endl; // ZZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" Cav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::cin >> sinputString;

    boundaryConditionsSetup(lattice, parameters, *boundaryCondition,
							xTub, yTub,yTubb,ymTubb, xCav, lCav, amplitude, frequence);

    // Main loop over time iterations.
    for (iT=iTMin; iT*parameters.getDeltaT()<=maxT; ++iT){//(plint iT=0; iT*parameters.getDeltaT()<maxT; ++iT) {
        // At this point, the state of the lattice corresponds to the
        //   discrete time iT. However, the stored averages (getStoredAverageEnergy
        //   and getStoredAverageDensity) correspond to the previous time iT-1.


        if (imSave>0) {
            if (iT%parameters.nStep(imSave)==0) {
            const T dt = parameters.getDeltaT();
            pcout << "Saving Gif ... at t =  " << iT*dt << endl;
            writeGif(lattice,parameters,gifFileIndex++);
            }}
        /*if (iT%parameters.nStep(imSave)==0) {
            pcout << "iT =  " << endl;
            const T dt = parameters.getDeltaT();
            pcout << "Saving Gif ... at t =  " << iT*dt << endl;
            writeGif(lattice,parameters, iT);
        }*/

        if (iT%parameters.nStep(vtkSave)==0 && iT>0) {
            pcout << "Saving VTK file ..." << endl;
            writeVTK(lattice, parameters, iT);
        }

        // Trace Vx et Vy
        if (iT%parameters.nStep(profileSave)==0) {
            // Measure in x dir along x-axis.
            // Definition of the section of track: in the middle of the tube, along the x-axis.
            //Box2D profileSectionXaxis(x0, x1, yM, yM);

			//Box2D profileSectionXaxis(xTub+xCav, xTub+xCav, yM, yM);  //ny for kortere fil, les density UK 020412
            Box2D profileSectionXaxis1(x1*0 , x1*0, yM, yM);

            //trace Vx along x-axis
            xAxisVxProfiles1
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis1, xComponent) )
                    << endl;



            densityProfiles1
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis1)
                << endl;

            Box2D profileSectionXaxis2(x1*0.02 , x1*0.02, yM, yM);

            densityProfiles2
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis2)
                << endl;

            Box2D profileSectionXaxis3(x1*0.04 , x1*0.04, yM, yM);

            densityProfiles3
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis3)
                << endl;
                Box2D profileSectionXaxis4(x1*0.06 , x1*0.06, yM, yM);

            densityProfiles4
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis4)
                << endl;
                Box2D profileSectionXaxis5(x1*0.08 , x1*0.08, yM, yM);

            densityProfiles5
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis5)
                << endl;
                Box2D profileSectionXaxis6(x1*0.10 , x1*0.10, yM, yM);

            densityProfiles6
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis6)
                << endl;
                Box2D profileSectionXaxis7(x1*0.12 , x1*0.12, yM, yM);

            densityProfiles7
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis7)
                << endl;
                Box2D profileSectionXaxis8(x1*0.14 , x1*0.14, yM, yM);

            densityProfiles8
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis8)
                << endl;
                Box2D profileSectionXaxis9(x1*0.16 , x1*0.16, yM, yM);

            densityProfiles9
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis9)
                << endl;
                Box2D profileSectionXaxis10(x1*0.18 , x1*0.18, yM, yM);

            densityProfiles10
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis10)
                << endl;

Box2D profileSectionXaxis11(x1*0.20 , x1*0.20, yM, yM);

            densityProfiles11
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis11)
                << endl;
Box2D profileSectionXaxis12(x1*0.22 , x1*0.22, yM, yM);

            densityProfiles12
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis12)
                << endl;
Box2D profileSectionXaxis13(x1*0.24 , x1*0.24, yM, yM);

            densityProfiles13
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis13)
                << endl;
Box2D profileSectionXaxis14(x1*0.26 , x1*0.26, yM, yM);

            densityProfiles14
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis14)
                << endl;
Box2D profileSectionXaxis15(x1*0.28 , x1*0.28, yM, yM);

            densityProfiles15
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis15)
                << endl;
Box2D profileSectionXaxis16(x1*0.30 , x1*0.30, yM, yM);

            densityProfiles16
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis16)
                << endl;
Box2D profileSectionXaxis17(x1*0.32 , x1*0.32, yM, yM);

            densityProfiles17
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis17)
                << endl;
Box2D profileSectionXaxis18(x1*0.34 , x1*0.34, yM, yM);

            densityProfiles18
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis18)
                << endl;
Box2D profileSectionXaxis19(x1*0.36 , x1*0.36, yM, yM);

            densityProfiles19
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis19)
                << endl;
Box2D profileSectionXaxis20(x1*0.38 , x1*0.38, yM, yM);

            densityProfiles20
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis20)
                << endl;
Box2D profileSectionXaxis21(x1*0.40 , x1*0.40, yM, yM);

            densityProfiles21
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis21)
                << endl;
Box2D profileSectionXaxis22(x1*0.42 , x1*0.42, yM, yM);

            densityProfiles22
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis22)
                << endl;
Box2D profileSectionXaxis23(x1*0.44 , x1*0.44, yM, yM);

            densityProfiles23
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis23)
                << endl;
Box2D profileSectionXaxis24(x1*0.46 , x1*0.46, yM, yM);

            densityProfiles24
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis24)
                << endl;
Box2D profileSectionXaxis25(x1*0.48 , x1*0.48, yM, yM);


            densityProfiles25
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis25)
                << endl;
Box2D profileSectionXaxis26(x1*0.50 , x1*0.50, yM, yM);

            densityProfiles26
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis26)
                << endl;
Box2D profileSectionXaxis27(x1*0.52 , x1*0.52, yM, yM);

            densityProfiles27
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis27)
                << endl;
Box2D profileSectionXaxis28(x1*0.54 , x1*0.54, yM, yM);

            densityProfiles28
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis28)
                << endl;
Box2D profileSectionXaxis29(x1*0.56 , x1*0.56, yM, yM);

            densityProfiles29
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis29)
                << endl;
Box2D profileSectionXaxis30(x1*0.58 , x1*0.58, yM, yM);

            densityProfiles30
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis30)
                << endl;
Box2D profileSectionXaxis31(x1*0.60 , x1*0.60, yM, yM);

            densityProfiles31
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis31)
                << endl;
Box2D profileSectionXaxis32(x1*0.62 , x1*0.62, yM, yM);

            densityProfiles32
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis32)
                << endl;
Box2D profileSectionXaxis33(x1*0.64 , x1*0.64, yM, yM);

            densityProfiles33
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis33)
                << endl;
Box2D profileSectionXaxis34(x1*0.66 , x1*0.66, yM, yM);

            densityProfiles34
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis34)
                << endl;
Box2D profileSectionXaxis35(x1*0.68 , x1*0.68, yM, yM);

            densityProfiles35
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis35)
                << endl;
Box2D profileSectionXaxis36(x1*0.70 , x1*0.70, yM, yM);

            densityProfiles36
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis36)
                << endl;
Box2D profileSectionXaxis37(x1*0.72 , x1*0.72, yM, yM);

            densityProfiles37
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis37)
                << endl;
Box2D profileSectionXaxis38(x1*0.74 , x1*0.74, yM, yM);

            densityProfiles38
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis38)
                << endl;
Box2D profileSectionXaxis39(x1*0.76 , x1*0.76, yM, yM);

            densityProfiles39
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis39)
                << endl;
Box2D profileSectionXaxis40(x1*0.78 , x1*0.78, yM, yM);

            densityProfiles40
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis40)
                << endl;
Box2D profileSectionXaxis41(x1*0.80 , x1*0.80, yM, yM);

            densityProfiles41
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis41)
                << endl;
Box2D profileSectionXaxis42(x1*0.82 , x1*0.82, yM, yM);

            densityProfiles42
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis42)
                << endl;
Box2D profileSectionXaxis43(x1*0.84 , x1*0.84, yM, yM);

            densityProfiles43
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis43)
                << endl;
Box2D profileSectionXaxis44(x1*0.86 , x1*0.86, yM, yM);

            densityProfiles44
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis44)
                << endl;
Box2D profileSectionXaxis45(x1*0.88 , x1*0.88, yM, yM);

            densityProfiles45
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis45)
                << endl;
Box2D profileSectionXaxis46(x1*0.90 , x1*0.90, yM, yM);

            densityProfiles46
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis46)
                << endl;
Box2D profileSectionXaxis47(x1*0.92 , x1*0.92, yM, yM);

            densityProfiles47
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis47)
                << endl;
Box2D profileSectionXaxis48(x1*0.94 , x1*0.94, yM, yM);

            densityProfiles48
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis48)
                << endl;
Box2D profileSectionXaxis49(x1*0.96 , x1*0.96, yM, yM);

            densityProfiles49
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis49)
                << endl;
Box2D profileSectionXaxis50(x1*0.98 , x1*0.98, yM, yM);
 densityProfiles50
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis50)
                << endl;
Box2D profileSectionXaxis51(x1*1 , x1*1, yM, yM);

 densityProfiles51
                << setprecision(6)
                << *computeDensity (lattice, profileSectionXaxis51)
                << endl;

                xAxisVxProfiles2
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis2, xComponent) )
                    << endl;
                     xAxisVxProfiles3
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis3, xComponent) )
                    << endl;
                     xAxisVxProfiles4
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis4, xComponent) )
                    << endl;
                     xAxisVxProfiles5
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis5, xComponent) )
                    << endl;
                     xAxisVxProfiles6
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis6, xComponent) )
                    << endl;
                     xAxisVxProfiles7
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis7, xComponent) )
                    << endl;
                     xAxisVxProfiles8
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis8, xComponent) )
                    << endl;
                     xAxisVxProfiles9
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis9, xComponent) )
                    << endl;
                     xAxisVxProfiles10
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis10, xComponent) )
                    << endl;
                     xAxisVxProfiles11
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis11, xComponent) )
                    << endl;
                     xAxisVxProfiles12
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis12, xComponent) )
                    << endl;
                     xAxisVxProfiles13
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis13, xComponent) )
                    << endl;
                     xAxisVxProfiles14
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis14, xComponent) )
                    << endl;
                     xAxisVxProfiles15
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis15, xComponent) )
                    << endl;
                     xAxisVxProfiles16
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis16, xComponent) )
                    << endl;
                     xAxisVxProfiles17
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis17, xComponent) )
                    << endl;
                     xAxisVxProfiles18
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis18, xComponent) )
                    << endl;
                     xAxisVxProfiles19
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis19, xComponent) )
                    << endl;
                     xAxisVxProfiles20
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis20, xComponent) )
                    << endl;
                     xAxisVxProfiles21
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis21, xComponent) )
                    << endl;
                     xAxisVxProfiles22
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis22, xComponent) )
                    << endl;
                     xAxisVxProfiles23
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis23, xComponent) )
                    << endl;
                     xAxisVxProfiles24
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis24, xComponent) )
                    << endl;
                     xAxisVxProfiles25
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis25, xComponent) )
                    << endl;
                     xAxisVxProfiles26
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis26, xComponent) )
                    << endl;
                     xAxisVxProfiles27
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis27, xComponent) )
                    << endl;
                     xAxisVxProfiles28
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis28, xComponent) )
                    << endl;
                     xAxisVxProfiles29
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis29, xComponent) )
                    << endl;
                     xAxisVxProfiles30
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis30, xComponent) )
                    << endl;
                     xAxisVxProfiles31
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis31, xComponent) )
                    << endl;
                     xAxisVxProfiles32
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis32, xComponent) )
                    << endl;
                     xAxisVxProfiles33
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis33, xComponent) )
                    << endl;
                     xAxisVxProfiles34
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis34, xComponent) )
                    << endl;
                     xAxisVxProfiles35
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis35, xComponent) )
                    << endl;
                     xAxisVxProfiles36
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis36, xComponent) )
                    << endl;
                     xAxisVxProfiles37
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis37, xComponent) )
                    << endl;
                     xAxisVxProfiles38
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis38, xComponent) )
                    << endl;
                     xAxisVxProfiles39
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis39, xComponent) )
                    << endl;
                     xAxisVxProfiles40
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis40, xComponent) )
                    << endl;
                     xAxisVxProfiles41
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis41, xComponent) )
                    << endl;
                     xAxisVxProfiles42
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis42, xComponent) )
                    << endl;
                     xAxisVxProfiles43
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis43, xComponent) )
                    << endl;
                     xAxisVxProfiles44
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis44, xComponent) )
                    << endl;
                     xAxisVxProfiles45
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis45, xComponent) )
                    << endl;
                    xAxisVxProfiles46
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis46, xComponent) )
                    << endl;
                     xAxisVxProfiles47
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis47, xComponent) )
                    << endl;
                     xAxisVxProfiles48
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis48, xComponent) )
                    << endl;
                    xAxisVxProfiles49
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis49, xComponent) )
                    << endl;
                     xAxisVxProfiles50
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis50, xComponent) )
                    << endl;
                     xAxisVxProfiles51
            << setprecision(4)
            // (2) Convert from lattice to physical units.
                    << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
                          *computeVelocityComponent (lattice, profileSectionXaxis51, xComponent) )
                    << endl;




/*
			// Profile 1
            // Definition of the section of track: the entrance of the tube
            Box2D profileSection1(xCav-rayon1, xCav-rayon1, y1+1, y2-1);
// What does this do?

            //trace Vx
            Vx1Profiles
                << setprecision(4)
                  // (2) Convert from lattice to physical units.
                << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                  // (1) Compute velocity norm along the chosen section.
                   *computeVelocityComponent (lattice, profileSection1, xComponent) )
                << endl;

            //trace Vy
            Vy1Profiles
                << setprecision(4)
                  // (2) Convert from lattice to physical units.
                << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                  // (1) Compute velocity norm along the chosen section.
                   *computeVelocityComponent (lattice, profileSection1, yComponent) )
                << endl;

			// Profile 2
            // Definition of the section of track: the output of the cavity
            Box2D profileSection2(xCav+lCav+rayon1, xCav+lCav+rayon1, y1+1, y2-1);

            //trace Vx
            Vx2Profiles
                << setprecision(4)
                  // (2) Convert from lattice to physical units.
                << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                  // (1) Compute velocity norm along the chosen section.
                   *computeVelocityComponent (lattice, profileSection2, xComponent) )
                << endl;

            //trace Vy
            Vy2Profiles
                << setprecision(4)
                  // (2) Convert from lattice to physical units.
                << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                  // (1) Compute velocity norm along the chosen section.
                   *computeVelocityComponent (lattice, profileSection2, yComponent) )
                << endl;
 */
	    }

    const T dx = parameters.getDeltaX();
    const T dt = parameters.getDeltaT();
 /*if ( (saveVorticity>0) && (iT*dt>=initVorticity) ) {
            if (iT%parameters.nStep(saveVorticity)==0) {
                // Definition du domaine du trace : ici tout le domaine
                Box2D domaine2d(boxVortx1,boxVortx2,boxVorty1,boxVorty2);

                //trace la vorticite
                Vorticity
                    << setprecision(8)
                    << *multiply ( dx/dt, *computeVorticity (*computeVelocity (lattice), domaine2d ) )
                    << endl;
            }}
*/
        if (iT%parameters.nStep(logT)==0) {
            Box2D profileSectionXaxis13(x1*0.24 , x1*0.24, yM, yM);
            pcout << "step " << iT
                  << "; t=" << iT*parameters.getDeltaT()
                  << "; delta X is " << parameters.getDeltaX()
                  << "; delta T is " << parameters.getDeltaT()
                  << "; deltaX/deltaT is " << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),

                          *computeVelocityComponent (lattice, profileSectionXaxis13, xComponent) )
                  << "; amplitude = " << amplitude
                  << "; frequence = " << frequence
                  << "; pulsation = " << pulsation((T)iT, amplitude, frequence, parameters)<<endl;
        }

        // Lattice Boltzmann iteration step.
        lattice.collideAndStream();

        // At this point, the state of the lattice corresponds to the
        //   discrete time iT+1, and the stored averages are upgraded to time iT.
        if (iT%parameters.nStep(logT)==0) {
            pcout << "; av energy ="
                  << setprecision(10) << getStoredAverageEnergy<T>(lattice)
                  << "; av rho ="
                  << getStoredAverageDensity<T>(lattice) << endl;
        }
    }

    delete boundaryCondition;
}
